# encoding: utf-8

"""Test suite for pptx.shapes.autoshape module."""

from __future__ import absolute_import, division, print_function, unicode_literals

import pytest

from pptx.dml.fill import FillFormat
from pptx.dml.line import LineFormat
from pptx.enum.shapes import MSO_SHAPE, MSO_SHAPE_TYPE
from pptx.oxml import parse_xml
from pptx.oxml.shapes.autoshape import CT_PresetGeometry2D, CT_Shape
from pptx.shapes.autoshape import Adjustment, AdjustmentCollection, AutoShapeType, Shape
from pptx.text.text import TextFrame

from ..oxml.unitdata.shape import (
    a_cNvSpPr,
    a_gd,
    a_prstGeom,
    an_avLst,
    an_nvSpPr,
    an_sp,
    an_spPr,
)
from ..unitutil.cxml import element, xml
from ..unitutil.mock import class_mock, instance_mock, property_mock


class DescribeAdjustment(object):
    def it_knows_its_effective_value(self, effective_val_fixture_):
        adjustment, expected_effective_value = effective_val_fixture_
        assert adjustment.effective_value == expected_effective_value

    # fixture --------------------------------------------------------

    def _effective_adj_val_cases():
        return [
            # no actual, effective should be determined by default value
            (50000, None, 0.5),
            # actual matches default
            (50000, 50000, 0.5),
            # actual is different than default
            (50000, 12500, 0.125),
            # actual is zero
            (50000, 0, 0.0),
            # negative default
            (-20833, None, -0.20833),
            # negative actual
            (-20833, -5678901, -56.78901),
        ]

    @pytest.fixture(params=_effective_adj_val_cases())
    def effective_val_fixture_(self, request):
        name = None
        def_val, actual, expected_effective_value = request.param
        adjustment = Adjustment(name, def_val, actual)
        return adjustment, expected_effective_value


class DescribeAdjustmentCollection(object):
    def it_should_load_default_adjustment_values(self, prstGeom_cases_):
        prstGeom, prst, expected = prstGeom_cases_
        adjustments = AdjustmentCollection(prstGeom)._adjustments
        actuals = tuple([(adj.name, adj.def_val) for adj in adjustments])
        assert len(adjustments) == len(expected)
        assert actuals == expected

    def it_should_load_adj_val_actuals_from_xml(self, load_adj_actuals_fixture_):
        prstGeom, expected_actuals, prstGeom_xml = load_adj_actuals_fixture_
        adjustments = AdjustmentCollection(prstGeom)._adjustments
        actual_actuals = dict([(a.name, a.actual) for a in adjustments])
        assert actual_actuals == expected_actuals

    def it_provides_normalized_effective_value_on_indexed_access(
        self, indexed_access_fixture_
    ):
        prstGeom, prst, expected_values = indexed_access_fixture_
        adjustments = AdjustmentCollection(prstGeom)
        actual_values = [adjustments[idx] for idx in range(len(adjustments))]
        assert actual_values == expected_values

    def it_should_update_actual_value_on_indexed_assignment(
        self, indexed_assignment_fixture_
    ):
        """
        Assignment to AdjustmentCollection[n] updates nth actual
        """
        adjs, idx, new_val, expected = indexed_assignment_fixture_
        adjs[idx] = new_val
        assert adjs._adjustments[idx].actual == expected

    def it_should_round_trip_indexed_assignment(self, adjustments):
        new_value = 0.375
        assert adjustments[0] != new_value
        # exercise ---------------------
        adjustments[0] = new_value
        # verify -----------------------
        assert adjustments[0] == new_value

    def it_should_raise_on_bad_index(self, adjustments):
        with pytest.raises(IndexError):
            adjustments[-6]
        with pytest.raises(IndexError):
            adjustments[6]
        with pytest.raises(TypeError):
            adjustments[0.0]
        with pytest.raises(TypeError):
            adjustments["0"]
        with pytest.raises(IndexError):
            adjustments[-6] = 1.0
        with pytest.raises(IndexError):
            adjustments[6] = 1.0
        with pytest.raises(TypeError):
            adjustments[0.0] = 1.0
        with pytest.raises(TypeError):
            adjustments["0"] = 1.0

    def it_should_raise_on_assigned_bad_value(self, adjustments):
        """
        AdjustmentCollection[n] = val raises on val is not number
        """
        with pytest.raises(ValueError):
            adjustments[0] = "1.0"

    def it_writes_adj_vals_to_xml_on_assignment(self, adjustments_with_prstGeom_):
        adjs, guides = adjustments_with_prstGeom_
        adjs[0] = 0.999
        adjs._prstGeom.rewrite_guides.assert_called_once_with(guides)

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def adjustments(self):
        prstGeom = a_prstGeom().with_nsdecls().with_prst("chevron").element
        return AdjustmentCollection(prstGeom)

    @pytest.fixture
    def adjustments_with_prstGeom_(self, request):
        prstGeom = a_prstGeom().with_nsdecls().with_prst("chevron").element
        adjustments = AdjustmentCollection(prstGeom)
        prstGeom_ = instance_mock(request, CT_PresetGeometry2D, name="prstGeom_")
        adjustments._prstGeom = prstGeom_
        guides = [("adj", 99900)]
        return adjustments, guides

    def _adj_actuals_cases():
        gd_bldr = a_gd().with_name("adj2").with_fmla("val 25000")
        avLst_bldr = an_avLst().with_child(gd_bldr)
        mathDivide_bldr = (
            a_prstGeom().with_nsdecls().with_prst("mathDivide").with_child(avLst_bldr)
        )

        gd_bldr = a_gd().with_name("adj").with_fmla("val 25000")
        avLst_bldr = an_avLst().with_child(gd_bldr)
        rect_bldr = a_prstGeom().with_nsdecls().with_prst("rect").with_child(avLst_bldr)

        gd_bldr_1 = a_gd().with_name("adj1").with_fmla("val 111")
        gd_bldr_2 = a_gd().with_name("adj2").with_fmla("val 222")
        gd_bldr_3 = a_gd().with_name("adj3").with_fmla("val 333")
        avLst_bldr = (
            an_avLst().with_child(gd_bldr_1).with_child(gd_bldr_2).with_child(gd_bldr_3)
        )
        wedgeRoundRectCallout_bldr = (
            a_prstGeom()
            .with_nsdecls()
            .with_prst("wedgeRoundRectCallout")
            .with_child(avLst_bldr)
        )

        return [
            # no adjustment values in xml or spec
            (a_prstGeom().with_nsdecls().with_prst("rect").with_child(an_avLst()), ()),
            # no adjustment values in xml, but some in spec
            (
                a_prstGeom().with_nsdecls().with_prst("circularArrow"),
                (
                    ("adj1", None),
                    ("adj2", None),
                    ("adj3", None),
                    ("adj4", None),
                    ("adj5", None),
                ),
            ),
            # adjustment value in xml but none in spec
            (rect_bldr, ()),
            # middle adjustment value in xml
            (mathDivide_bldr, (("adj1", None), ("adj2", 25000), ("adj3", None))),
            # all adjustment values in xml
            (wedgeRoundRectCallout_bldr, (("adj1", 111), ("adj2", 222), ("adj3", 333))),
        ]

    @pytest.fixture(params=_adj_actuals_cases())
    def load_adj_actuals_fixture_(self, request):
        prstGeom_bldr, adj_vals = request.param
        prstGeom = prstGeom_bldr.element
        expected = dict(adj_vals)
        prstGeom_xml = prstGeom_bldr.xml
        return prstGeom, expected, prstGeom_xml

    def _prstGeom_cases():
        return [
            # rect has no adjustments
            ("rect", ()),
            # chevron has one simple one
            ("chevron", (("adj", 50000),)),
            # one with several and some negative
            (
                "accentBorderCallout1",
                (("adj1", 18750), ("adj2", -8333), ("adj3", 112500), ("adj4", -38333)),
            ),
            # another one with some negative
            (
                "wedgeRoundRectCallout",
                (("adj1", -20833), ("adj2", 62500), ("adj3", 16667)),
            ),
            # one with values outside normal range
            (
                "circularArrow",
                (
                    ("adj1", 12500),
                    ("adj2", 1142319),
                    ("adj3", 20457681),
                    ("adj4", 10800000),
                    ("adj5", 12500),
                ),
            ),
        ]

    @pytest.fixture(params=_prstGeom_cases())
    def prstGeom_cases_(self, request):
        prst, expected_values = request.param
        prstGeom = (
            a_prstGeom().with_nsdecls().with_prst(prst).with_child(an_avLst()).element
        )
        return prstGeom, prst, expected_values

    def _effective_val_cases():
        return [
            ("rect", ()),
            ("chevron", (0.5,)),
            ("circularArrow", (0.125, 11.42319, 204.57681, 108.0, 0.125)),
        ]

    @pytest.fixture(params=_effective_val_cases())
    def indexed_access_fixture_(self, request):
        prst, effective_values = request.param
        prstGeom = a_prstGeom().with_nsdecls().with_prst(prst).element
        return prstGeom, prst, list(effective_values)

    def _indexed_assignment_cases():
        return [("chevron", 0, 0.5, 50000), ("circularArrow", 4, 99.99, 9999000)]

    @pytest.fixture(params=_indexed_assignment_cases())
    def indexed_assignment_fixture_(self, request):
        prst, idx, new_val, expected = request.param
        prstGeom = a_prstGeom().with_nsdecls().with_prst(prst).element
        adjustments = AdjustmentCollection(prstGeom)
        return adjustments, idx, new_val, expected


class DescribeAutoShapeType(object):
    def it_knows_the_details_of_the_auto_shape_type_it_represents(self):
        autoshape_type = AutoShapeType(MSO_SHAPE.ROUNDED_RECTANGLE)
        assert autoshape_type.autoshape_type_id == MSO_SHAPE.ROUNDED_RECTANGLE
        assert autoshape_type.prst == "roundRect"
        assert autoshape_type.basename == "Rounded Rectangle"

    def it_knows_the_default_adj_vals_for_its_autoshape_type(
        self, default_adj_vals_fixture_
    ):
        prst, default_adj_vals = default_adj_vals_fixture_
        _default_adj_vals = AutoShapeType.default_adjustment_values(prst)
        assert _default_adj_vals == default_adj_vals

    def it_knows_the_autoshape_type_id_for_each_prst_key(self):
        assert AutoShapeType.id_from_prst("rect") == MSO_SHAPE.RECTANGLE

    def it_raises_when_asked_for_autoshape_type_id_with_a_bad_prst(self):
        with pytest.raises(KeyError):
            AutoShapeType.id_from_prst("badPrst")

    def it_caches_autoshape_type_lookups(self):
        autoshape_type_id = MSO_SHAPE.ROUNDED_RECTANGLE
        autoshape_type_1 = AutoShapeType(autoshape_type_id)
        autoshape_type_2 = AutoShapeType(autoshape_type_id)
        assert autoshape_type_2 is autoshape_type_1

    def it_raises_on_construction_with_bad_autoshape_type_id(self):
        with pytest.raises(KeyError):
            AutoShapeType(9999)

    # fixtures -------------------------------------------------------

    def _default_adj_vals_cases():
        return [
            (MSO_SHAPE.RECTANGLE, ()),
            (MSO_SHAPE.CHEVRON, (("adj", 50000),)),
            (
                MSO_SHAPE.LEFT_CIRCULAR_ARROW,
                (
                    ("adj1", 12500),
                    ("adj2", -1142319),
                    ("adj3", 1142319),
                    ("adj4", 10800000),
                    ("adj5", 12500),
                ),
            ),
        ]

    @pytest.fixture(params=_default_adj_vals_cases())
    def default_adj_vals_fixture_(self, request):
        prst, default_adj_vals = request.param
        return prst, default_adj_vals


class DescribeShape(object):
    """Unit-test suite for `pptx.shapes.autoshape.Shape` object."""

    def it_initializes_adjustments_on_first_ref(self, init_adjs_fixture_):
        shape, adjs_, AdjustmentCollection_, sp_ = init_adjs_fixture_
        assert shape.adjustments is adjs_
        AdjustmentCollection_.assert_called_once_with(sp_.prstGeom)

    def it_knows_its_autoshape_type(self, autoshape_type_fixture_):
        shape, expected_value = autoshape_type_fixture_
        auto_shape_type = shape.auto_shape_type
        assert auto_shape_type == expected_value

    def but_it_raises_when_auto_shape_type_called_on_non_autoshape(
        self, non_autoshape_shape_
    ):
        with pytest.raises(ValueError):
            non_autoshape_shape_.auto_shape_type

    def it_has_a_fill(self, shape):
        assert isinstance(shape.fill, FillFormat)

    def it_knows_it_has_a_text_frame(self):
        shape = Shape(None, None)
        assert shape.has_text_frame is True

    def it_has_a_line(self, shape):
        assert isinstance(shape.line, LineFormat)

    def it_knows_its_shape_type_when_its_a_placeholder(self, placeholder_shape_):
        assert placeholder_shape_.shape_type == MSO_SHAPE_TYPE.PLACEHOLDER

    def and_it_knows_its_shape_type_when_its_not_a_placeholder(
        self, non_placeholder_shapes_
    ):
        autoshape_shape_, textbox_shape_, freeform_ = non_placeholder_shapes_
        assert autoshape_shape_.shape_type == MSO_SHAPE_TYPE.AUTO_SHAPE
        assert textbox_shape_.shape_type == MSO_SHAPE_TYPE.TEXT_BOX
        assert freeform_.shape_type == MSO_SHAPE_TYPE.FREEFORM

    def but_it_raises_when_shape_type_called_on_unrecognized_shape(self):
        xml_ = (
            '<p:sp xmlns:p="http://schemas.openxmlformats.org/presentationml/'
            '2006/main" xmlns:a="http://schemas.openxmlformats.org/drawingml/'
            '2006/main"><p:nvSpPr><p:cNvPr id="9" name="Unknown Shape Type 8"'
            "/><p:cNvSpPr/><p:nvPr/></p:nvSpPr><p:spPr/></p:sp>"
        )
        sp = parse_xml(xml_)
        shape = Shape(sp, None)
        # verify -----------------------
        with pytest.raises(NotImplementedError):
            shape.shape_type

    def it_knows_what_text_it_contains(self, text_frame_prop_, text_frame_):
        text_frame_prop_.return_value = text_frame_
        text_frame_.text = "foobar"
        shape = Shape(None, None)

        text = shape.text

        assert text == "foobar"

    def it_can_change_its_text(self, text_frame_prop_, text_frame_):
        text_frame_prop_.return_value = text_frame_
        shape = Shape(None, None)

        shape.text = "føøbår"

        assert text_frame_.text == "føøbår"

    def it_provides_access_to_its_text_frame(self, text_frame_fixture):
        shape, text_frame_, TextFrame_, txBody = text_frame_fixture
        text_frame = shape.text_frame
        TextFrame_.assert_called_once_with(txBody, shape)
        assert text_frame is text_frame_

    def and_it_creates_a_txBody_if_needed(self, txBody_fixture):
        shape, expected_xml = txBody_fixture
        text_frame = shape.text_frame
        assert text_frame._txBody.xml == expected_xml

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def autoshape_type_fixture_(self, shape, prst):
        sp = (
            an_sp()
            .with_nsdecls()
            .with_child(an_nvSpPr().with_child(a_cNvSpPr()))
            .with_child(an_spPr().with_child(a_prstGeom().with_prst("chevron")))
        ).element
        shape = Shape(sp, None)
        return shape, MSO_SHAPE.CHEVRON

    @pytest.fixture
    def init_adjs_fixture_(
        self, request, shape, sp_, adjustments_, AdjustmentCollection_
    ):
        return shape, adjustments_, AdjustmentCollection_, sp_

    @pytest.fixture
    def text_frame_fixture(self, TextFrame_, text_frame_):
        sp = element("p:sp/p:txBody")
        txBody = sp[0]
        shape = Shape(sp, None)
        return shape, text_frame_, TextFrame_, txBody

    @pytest.fixture
    def txBody_fixture(self, request):
        sp_cxml = "p:sp"
        expected_cxml = "p:txBody/(a:bodyPr,a:p)"
        shape = Shape(element(sp_cxml), None)
        expected_xml = xml(expected_cxml)
        return shape, expected_xml

    # fixture components ---------------------------------------------

    @pytest.fixture
    def AdjustmentCollection_(self, request, adjustments_):
        return class_mock(
            request,
            "pptx.shapes.autoshape.AdjustmentCollection",
            return_value=adjustments_,
        )

    @pytest.fixture
    def adjustments_(self, request):
        return instance_mock(request, AdjustmentCollection)

    @pytest.fixture
    def non_autoshape_shape_(self, request, sp_):
        sp_.is_autoshape = False
        return Shape(sp_, None)

    @pytest.fixture
    def non_placeholder_shapes_(self, request):
        autoshape_sp_ = instance_mock(
            request,
            CT_Shape,
            name="autoshape_sp_",
            is_autoshape=True,
            is_textbox=False,
            has_custom_geometry=False,
        )
        autoshape_shape_ = Shape(autoshape_sp_, None)
        textbox_sp_ = instance_mock(
            request,
            CT_Shape,
            name="textbox_sp_",
            is_autoshape=False,
            is_textbox=True,
            has_custom_geometry=False,
        )
        textbox_shape_ = Shape(textbox_sp_, None)
        freeform_sp_ = instance_mock(
            request,
            CT_Shape,
            name="freeform_sp_",
            is_autoshape=True,
            is_textbox=False,
            has_custom_geometry=True,
        )
        freeform_ = Shape(freeform_sp_, None)
        property_mock(request, Shape, "is_placeholder", return_value=False)
        return autoshape_shape_, textbox_shape_, freeform_

    @pytest.fixture
    def placeholder_shape_(self, request, sp_):
        placeholder_shape_ = Shape(sp_, None)
        property_mock(request, Shape, "is_placeholder", return_value=True)
        return placeholder_shape_

    @pytest.fixture
    def prst(self):
        return MSO_SHAPE.CHEVRON

    @pytest.fixture
    def shape(self, request, sp_):
        return Shape(sp_, None)

    @pytest.fixture
    def sp_(self, request, prst):
        return instance_mock(request, CT_Shape, prst=prst, is_autoshape=True)

    @pytest.fixture
    def TextFrame_(self, request, text_frame_):
        return class_mock(
            request, "pptx.shapes.autoshape.TextFrame", return_value=text_frame_
        )

    @pytest.fixture
    def text_frame_(self, request):
        return instance_mock(request, TextFrame)

    @pytest.fixture
    def text_frame_prop_(self, request):
        return property_mock(request, Shape, "text_frame")



# encoding: utf-8

"""Unit test suite for pptx.parts.image module."""

from __future__ import absolute_import, division, print_function, unicode_literals

import pytest

from pptx.compat import BytesIO
from pptx.package import Package
from pptx.parts.image import Image, ImagePart
from pptx.util import Emu

from ..unitutil.file import absjoin, test_file_dir
from ..unitutil.mock import (
    class_mock,
    initializer_mock,
    instance_mock,
    method_mock,
    property_mock,
)


images_pptx_path = absjoin(test_file_dir, "with_images.pptx")

test_image_path = absjoin(test_file_dir, "python-icon.jpeg")
test_eps_path = absjoin(test_file_dir, "cdw-logo.eps")
new_image_path = absjoin(test_file_dir, "monty-truth.png")


class DescribeImagePart(object):
    def it_can_construct_from_an_image_object(self, new_fixture):
        package_, image_, _init_, partname_ = new_fixture

        image_part = ImagePart.new(package_, image_)

        package_.next_image_partname.assert_called_once_with(image_.ext)
        _init_.assert_called_once_with(
            partname_, image_.content_type, image_.blob, package_, image_.filename
        )
        assert isinstance(image_part, ImagePart)

    def it_provides_access_to_its_image(self, image_fixture):
        image_part, Image_, blob, desc, image_ = image_fixture
        image = image_part.image
        Image_.assert_called_once_with(blob, desc)
        assert image is image_

    def it_can_scale_its_dimensions(self, scale_fixture):
        image_part, width, height, expected_values = scale_fixture
        assert image_part.scale(width, height) == expected_values

    def it_knows_its_pixel_dimensions(self, size_fixture):
        image, expected_size = size_fixture
        assert image._px_size == expected_size

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def image_fixture(self, Image_, image_):
        blob, filename = "blob", "foobar.png"
        image_part = ImagePart(None, None, blob, None, filename)
        return image_part, Image_, blob, filename, image_

    @pytest.fixture
    def new_fixture(self, request, package_, image_, _init_):
        partname_ = package_.next_image_partname.return_value
        return package_, image_, _init_, partname_

    @pytest.fixture(
        params=[
            (None, None, Emu(2590800), Emu(2590800)),
            (1000, None, 1000, 1000),
            (None, 3000, 3000, 3000),
            (3337, 9999, 3337, 9999),
        ]
    )
    def scale_fixture(self, request):
        width, height, expected_width, expected_height = request.param
        with open(test_image_path, "rb") as f:
            blob = f.read()
        image = ImagePart(None, None, blob, None)
        return image, width, height, (expected_width, expected_height)

    @pytest.fixture
    def size_fixture(self):
        with open(test_image_path, "rb") as f:
            blob = f.read()
        image = ImagePart(None, None, blob, None)
        return image, (204, 204)

    # fixture components ---------------------------------------------

    @pytest.fixture
    def Image_(self, request, image_):
        return class_mock(request, "pptx.parts.image.Image", return_value=image_)

    @pytest.fixture
    def image_(self, request):
        return instance_mock(request, Image)

    @pytest.fixture
    def _init_(self, request):
        return initializer_mock(request, ImagePart)

    @pytest.fixture
    def package_(self, request):
        return instance_mock(request, Package)


class DescribeImage(object):
    def it_can_construct_from_a_path(self, from_path_fixture):
        image_file, blob, filename, image_ = from_path_fixture
        image = Image.from_file(image_file)
        Image.from_blob.assert_called_once_with(blob, filename)
        assert image is image_

    def it_can_construct_from_a_stream(self, from_stream_fixture):
        image_file, blob, image_ = from_stream_fixture
        image = Image.from_file(image_file)
        Image.from_blob.assert_called_once_with(blob, None)
        assert image is image_

    def it_can_construct_from_a_blob(self, from_blob_fixture):
        blob, filename = from_blob_fixture
        image = Image.from_blob(blob, filename)
        Image.__init__.assert_called_once_with(blob, filename)
        assert isinstance(image, Image)

    def it_knows_its_blob(self, blob_fixture):
        image, expected_value = blob_fixture
        assert image.blob == expected_value

    def it_knows_its_content_type(self, content_type_fixture):
        image, expected_value = content_type_fixture
        assert image.content_type == expected_value

    def it_knows_its_canonical_filename_extension(self, ext_fixture):
        image, expected_value = ext_fixture
        assert image.ext == expected_value

    def it_knows_its_dpi(self, dpi_fixture):
        image, expected_value = dpi_fixture
        assert image.dpi == expected_value

    def it_knows_its_filename(self, filename_fixture):
        image, expected_value = filename_fixture
        assert image.filename == expected_value

    def it_knows_its_sha1_hash(self):
        image = Image(b"foobar", None)
        assert image.sha1 == "8843d7f92416211de9ebb963ff4ce28125932878"

    def it_knows_its_PIL_properties_to_help(self, pil_fixture):
        image, size, format, dpi = pil_fixture
        assert image.size == size
        assert image._format == format
        assert image.dpi == dpi
        assert image._pil_props == (format, size, None)

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def blob_fixture(self):
        blob = b"foobar"
        image = Image(blob, None)
        return image, blob

    @pytest.fixture(
        params=[
            ("BMP", "image/bmp"),
            ("GIF", "image/gif"),
            ("JPEG", "image/jpeg"),
            ("PNG", "image/png"),
            ("TIFF", "image/tiff"),
            ("WMF", "image/x-wmf"),
        ]
    )
    def content_type_fixture(self, request, _format_):
        format, expected_value = request.param
        image = Image(None, None)
        _format_.return_value = format
        return image, expected_value

    @pytest.fixture(
        params=[
            ((42, 24), (42, 24)),
            ((42.1, 23.6), (42, 24)),
            (None, (72, 72)),
            ((3047, 2388), (72, 72)),
            ("foobar", (72, 72)),
        ]
    )
    def dpi_fixture(self, request, _pil_props_):
        raw_dpi, expected_dpi = request.param
        image = Image(None, None)
        _pil_props_.return_value = (None, None, raw_dpi)
        return image, expected_dpi

    @pytest.fixture(
        params=[
            ("BMP", "bmp"),
            ("GIF", "gif"),
            ("JPEG", "jpg"),
            ("PNG", "png"),
            ("TIFF", "tiff"),
            ("WMF", "wmf"),
        ]
    )
    def ext_fixture(self, request, _format_):
        format, expected_value = request.param
        image = Image(None, None)
        _format_.return_value = format
        return image, expected_value

    @pytest.fixture(params=["foo.bar", None])
    def filename_fixture(self, request):
        filename = request.param
        image = Image(None, filename)
        return image, filename

    @pytest.fixture
    def from_blob_fixture(self, _init_):
        blob, filename = b"foobar", "foo.png"
        return blob, filename

    @pytest.fixture
    def from_path_fixture(self, from_blob_, image_):
        image_file = test_image_path
        with open(test_image_path, "rb") as f:
            blob = f.read()
        filename = "python-icon.jpeg"
        from_blob_.return_value = image_
        return image_file, blob, filename, image_

    @pytest.fixture
    def from_stream_fixture(self, from_blob_, image_):
        with open(test_image_path, "rb") as f:
            blob = f.read()
            image_file = BytesIO(blob)
        from_blob_.return_value = image_
        return image_file, blob, image_

    @pytest.fixture
    def pil_fixture(self):
        with open(test_image_path, "rb") as f:
            blob = f.read()
        image = Image(blob, None)
        size = (204, 204)
        format = "JPEG"
        dpi = (72, 72)
        return image, size, format, dpi

    # fixture components ---------------------------------------------

    @pytest.fixture
    def _format_(self, request):
        return property_mock(request, Image, "_format")

    @pytest.fixture
    def from_blob_(self, request):
        return method_mock(request, Image, "from_blob")

    @pytest.fixture
    def image_(self, request):
        return instance_mock(request, Image)

    @pytest.fixture
    def _init_(self, request):
        return initializer_mock(request, Image)

    @pytest.fixture
    def _pil_props_(self, request):
        return property_mock(request, Image, "_pil_props")



# encoding: utf-8

"""
Test suite for pptx.parts.presentation module
"""

from __future__ import absolute_import, print_function

import pytest

from pptx.opc.constants import RELATIONSHIP_TYPE as RT
from pptx.opc.packuri import PackURI
from pptx.package import Package
from pptx.parts.coreprops import CorePropertiesPart
from pptx.parts.presentation import PresentationPart
from pptx.parts.slide import NotesMasterPart, SlidePart
from pptx.presentation import Presentation
from pptx.slide import NotesMaster, Slide, SlideLayout, SlideMaster

from ..unitutil.cxml import element
from ..unitutil.mock import call, class_mock, instance_mock, method_mock, property_mock


class DescribePresentationPart(object):
    def it_provides_access_to_its_presentation(self, prs_fixture):
        prs_part, Presentation_, prs_elm, prs_ = prs_fixture
        prs = prs_part.presentation
        Presentation_.assert_called_once_with(prs_elm, prs_part)
        assert prs is prs_

    def it_provides_access_to_its_core_properties(self, core_props_fixture):
        prs_part, core_properties_ = core_props_fixture
        core_properties = prs_part.core_properties
        assert core_properties is core_properties_

    def it_provides_access_to_the_notes_master_part(self, nmp_get_fixture):
        """
        This is the first of a two-part test to cover the existing notes
        master case. The notes master not-present case follows.
        """
        prs_part, notes_master_part_ = nmp_get_fixture
        notes_master_part = prs_part.notes_master_part
        prs_part.part_related_by.assert_called_once_with(prs_part, RT.NOTES_MASTER)
        assert notes_master_part is notes_master_part_

    def it_adds_a_notes_master_part_when_needed(self, nmp_add_fixture):
        """
        This is the second of a two-part test to cover the
        notes-master-not-present case. The notes master present case is just
        above.
        """
        prs_part, NotesMasterPart_ = nmp_add_fixture[:2]
        package_, notes_master_part_ = nmp_add_fixture[2:]

        notes_master_part = prs_part.notes_master_part

        NotesMasterPart_.create_default.assert_called_once_with(package_)
        prs_part.relate_to.assert_called_once_with(
            prs_part, notes_master_part_, RT.NOTES_MASTER
        )
        assert notes_master_part is notes_master_part_

    def it_provides_access_to_its_notes_master(self, notes_master_fixture):
        prs_part, notes_master_ = notes_master_fixture
        notes_master = prs_part.notes_master
        assert notes_master is notes_master_

    def it_provides_access_to_a_related_slide(self, slide_fixture):
        prs_part, rId, slide_ = slide_fixture
        slide = prs_part.related_slide(rId)
        prs_part.related_parts.__getitem__.assert_called_once_with(rId)
        assert slide is slide_

    def it_provides_access_to_a_related_master(self, master_fixture):
        prs_part, rId, slide_master_ = master_fixture
        slide_master = prs_part.related_slide_master(rId)
        prs_part.related_parts.__getitem__.assert_called_once_with(rId)
        assert slide_master is slide_master_

    def it_can_rename_related_slide_parts(self, rename_fixture):
        prs_part, rIds, getitem_ = rename_fixture[:3]
        calls, slide_parts, expected_names = rename_fixture[3:]
        prs_part.rename_slide_parts(rIds)
        assert getitem_.call_args_list == calls
        assert [sp.partname for sp in slide_parts] == expected_names

    def it_can_save_the_package_to_a_file(self, save_fixture):
        prs_part, file_, package_ = save_fixture
        prs_part.save(file_)
        package_.save.assert_called_once_with(file_)

    def it_can_add_a_new_slide(self, add_slide_fixture):
        prs_part, slide_layout_, SlidePart_, partname = add_slide_fixture[:4]
        package_, slide_layout_part_, slide_part_ = add_slide_fixture[4:7]
        rId_, slide_ = add_slide_fixture[7:]

        rId, slide = prs_part.add_slide(slide_layout_)

        SlidePart_.new.assert_called_once_with(partname, package_, slide_layout_part_)
        prs_part.relate_to.assert_called_once_with(prs_part, slide_part_, RT.SLIDE)
        assert rId is rId_
        assert slide is slide_

    def it_finds_the_slide_id_of_a_slide_part(self, slide_id_fixture):
        prs_part, slide_part_, expected_value = slide_id_fixture
        _slide_id = prs_part.slide_id(slide_part_)
        assert _slide_id == expected_value

    def it_raises_on_slide_id_not_found(self, slide_id_raises_fixture):
        prs_part, slide_part_ = slide_id_raises_fixture
        with pytest.raises(ValueError):
            prs_part.slide_id(slide_part_)

    def it_finds_a_slide_by_slide_id(self, get_slide_fixture):
        prs_part, slide_id, expected_value = get_slide_fixture
        slide = prs_part.get_slide(slide_id)
        assert slide == expected_value

    def it_knows_the_next_slide_partname_to_help(self, next_fixture):
        prs_part, partname = next_fixture
        assert prs_part._next_slide_partname == partname

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def add_slide_fixture(
        self,
        package_,
        slide_layout_,
        SlidePart_,
        slide_part_,
        slide_,
        _next_slide_partname_prop_,
        relate_to_,
    ):
        prs_part = PresentationPart(None, None, None, package_)
        partname = _next_slide_partname_prop_.return_value
        rId_ = "rId42"
        SlidePart_.new.return_value = slide_part_
        relate_to_.return_value = rId_
        slide_layout_part_ = slide_layout_.part
        slide_part_.slide = slide_
        return (
            prs_part,
            slide_layout_,
            SlidePart_,
            partname,
            package_,
            slide_layout_part_,
            slide_part_,
            rId_,
            slide_,
        )

    @pytest.fixture
    def core_props_fixture(self, package_, core_properties_):
        prs_part = PresentationPart(None, None, None, package_)
        package_.core_properties = core_properties_
        return prs_part, core_properties_

    @pytest.fixture(params=[True, False])
    def get_slide_fixture(self, request, slide_, slide_part_, related_parts_prop_):
        is_present = request.param
        prs_elm = element(
            "p:presentation/p:sldIdLst/(p:sldId{r:id=a,id=256},p:sldId{r:id="
            "b,id=257},p:sldId{r:id=c,id=258})"
        )
        prs_part = PresentationPart(None, None, prs_elm)
        slide_id = 257 if is_present else 666
        expected_value = slide_ if is_present else None
        related_parts_prop_.return_value = {"a": None, "b": slide_part_, "c": None}
        slide_part_.slide = slide_
        return prs_part, slide_id, expected_value

    @pytest.fixture
    def master_fixture(self, slide_master_, related_parts_prop_):
        prs_part = PresentationPart(None, None, None, None)
        rId = "rId42"
        related_parts_ = related_parts_prop_.return_value
        related_parts_.__getitem__.return_value.slide_master = slide_master_
        return prs_part, rId, slide_master_

    @pytest.fixture
    def next_fixture(self):
        prs_elm = element("p:presentation/p:sldIdLst/(p:sldId,p:sldId)")
        prs_part = PresentationPart(None, None, prs_elm)
        partname = PackURI("/ppt/slides/slide3.xml")
        return prs_part, partname

    @pytest.fixture
    def notes_master_fixture(
        self, notes_master_part_prop_, notes_master_part_, notes_master_
    ):
        prs_part = PresentationPart(None, None, None, None)
        notes_master_part_prop_.return_value = notes_master_part_
        notes_master_part_.notes_master = notes_master_
        return prs_part, notes_master_

    @pytest.fixture
    def nmp_add_fixture(
        self,
        package_,
        NotesMasterPart_,
        notes_master_part_,
        part_related_by_,
        relate_to_,
    ):
        prs_part = PresentationPart(None, None, None, package_)
        part_related_by_.side_effect = KeyError
        NotesMasterPart_.create_default.return_value = notes_master_part_
        return prs_part, NotesMasterPart_, package_, notes_master_part_

    @pytest.fixture
    def nmp_get_fixture(self, notes_master_part_, part_related_by_):
        prs_part = PresentationPart(None, None, None, None)
        part_related_by_.return_value = notes_master_part_
        return prs_part, notes_master_part_

    @pytest.fixture
    def prs_fixture(self, Presentation_, prs_):
        prs_elm = element("p:presentation")
        prs_part = PresentationPart(None, None, prs_elm)
        return prs_part, Presentation_, prs_elm, prs_

    @pytest.fixture
    def rename_fixture(self, related_parts_prop_):
        prs_part = PresentationPart(None, None, None)
        rIds = ("rId1", "rId2")
        getitem_ = related_parts_prop_.return_value.__getitem__
        calls = [call("rId1"), call("rId2")]
        slide_parts = [SlidePart(None, None, None), SlidePart(None, None, None)]
        expected_names = [
            PackURI("/ppt/slides/slide1.xml"),
            PackURI("/ppt/slides/slide2.xml"),
        ]
        getitem_.side_effect = slide_parts
        return (prs_part, rIds, getitem_, calls, slide_parts, expected_names)

    @pytest.fixture
    def save_fixture(self, package_):
        prs_part = PresentationPart(None, None, None, package_)
        file_ = "foobar.docx"
        return prs_part, file_, package_

    @pytest.fixture
    def slide_fixture(self, slide_, related_parts_prop_):
        prs_part = PresentationPart(None, None, None, None)
        rId = "rId42"
        related_parts_ = related_parts_prop_.return_value
        related_parts_.__getitem__.return_value.slide = slide_
        return prs_part, rId, slide_

    @pytest.fixture
    def slide_id_fixture(self, slide_part_, related_parts_prop_):
        prs_elm = element(
            "p:presentation/p:sldIdLst/(p:sldId{r:id=a,id=256},p:sldId{r:id="
            "b,id=257},p:sldId{r:id=c,id=258})"
        )
        prs_part = PresentationPart(None, None, prs_elm)
        expected_value = 257
        related_parts_prop_.return_value = {"a": None, "b": slide_part_, "c": None}
        return prs_part, slide_part_, expected_value

    @pytest.fixture
    def slide_id_raises_fixture(self, slide_part_, related_parts_prop_):
        prs_elm = element(
            "p:presentation/p:sldIdLst/(p:sldId{r:id=a,id=256},p:sldId{r:id="
            "b,id=257},p:sldId{r:id=c,id=258})"
        )
        prs_part = PresentationPart(None, None, prs_elm)
        related_parts_prop_.return_value = {"a": None, "b": None, "c": None}
        return prs_part, slide_part_

    # fixture components ---------------------------------------------

    @pytest.fixture
    def core_properties_(self, request):
        return instance_mock(request, CorePropertiesPart)

    @pytest.fixture
    def _next_slide_partname_prop_(self, request):
        return property_mock(request, PresentationPart, "_next_slide_partname")

    @pytest.fixture
    def NotesMasterPart_(self, request, prs_):
        return class_mock(request, "pptx.parts.presentation.NotesMasterPart")

    @pytest.fixture
    def notes_master_(self, request):
        return instance_mock(request, NotesMaster)

    @pytest.fixture
    def notes_master_part_(self, request):
        return instance_mock(request, NotesMasterPart)

    @pytest.fixture
    def notes_master_part_prop_(self, request, notes_master_part_):
        return property_mock(request, PresentationPart, "notes_master_part")

    @pytest.fixture
    def package_(self, request):
        return instance_mock(request, Package)

    @pytest.fixture
    def part_related_by_(self, request):
        return method_mock(request, PresentationPart, "part_related_by", autospec=True)

    @pytest.fixture
    def Presentation_(self, request, prs_):
        return class_mock(
            request, "pptx.parts.presentation.Presentation", return_value=prs_
        )

    @pytest.fixture
    def prs_(self, request):
        return instance_mock(request, Presentation)

    @pytest.fixture
    def relate_to_(self, request):
        return method_mock(request, PresentationPart, "relate_to", autospec=True)

    @pytest.fixture
    def related_parts_prop_(self, request):
        return property_mock(request, PresentationPart, "related_parts")

    @pytest.fixture
    def slide_(self, request):
        return instance_mock(request, Slide)

    @pytest.fixture
    def slide_layout_(self, request):
        return instance_mock(request, SlideLayout)

    @pytest.fixture
    def slide_master_(self, request):
        return instance_mock(request, SlideMaster)

    @pytest.fixture
    def slide_part_(self, request):
        return instance_mock(request, SlidePart)

    @pytest.fixture
    def SlidePart_(self, request):
        return class_mock(request, "pptx.parts.presentation.SlidePart")



# encoding: utf-8

"""The shape tree, the structure that holds a slide's shapes."""

import os

from pptx.compat import BytesIO
from pptx.enum.shapes import PP_PLACEHOLDER, PROG_ID
from pptx.media import SPEAKER_IMAGE_BYTES, Video
from pptx.opc.constants import CONTENT_TYPE as CT
from pptx.oxml.ns import qn
from pptx.oxml.shapes.graphfrm import CT_GraphicalObjectFrame
from pptx.oxml.shapes.picture import CT_Picture
from pptx.oxml.simpletypes import ST_Direction
from pptx.shapes.autoshape import AutoShapeType, Shape
from pptx.shapes.base import BaseShape
from pptx.shapes.connector import Connector
from pptx.shapes.freeform import FreeformBuilder
from pptx.shapes.graphfrm import GraphicFrame
from pptx.shapes.group import GroupShape
from pptx.shapes.picture import Movie, Picture
from pptx.shapes.placeholder import (
    ChartPlaceholder,
    LayoutPlaceholder,
    MasterPlaceholder,
    NotesSlidePlaceholder,
    PicturePlaceholder,
    PlaceholderGraphicFrame,
    PlaceholderPicture,
    SlidePlaceholder,
    TablePlaceholder,
)
from pptx.shared import ParentedElementProxy
from pptx.util import Emu, lazyproperty

# +-- _BaseShapes
# |   |
# |   +-- _BaseGroupShapes
# |   |   |
# |   |   +-- GroupShapes
# |   |   |
# |   |   +-- SlideShapes
# |   |
# |   +-- LayoutShapes
# |   |
# |   +-- MasterShapes
# |   |
# |   +-- NotesSlideShapes
# |   |
# |   +-- BasePlaceholders
# |       |
# |       +-- LayoutPlaceholders
# |       |
# |       +-- MasterPlaceholders
# |           |
# |           +-- NotesSlidePlaceholders
# |
# +-- SlidePlaceholders


class _BaseShapes(ParentedElementProxy):
    """
    Base class for a shape collection appearing in a slide-type object,
    include Slide, SlideLayout, and SlideMaster, providing common methods.
    """

    def __init__(self, spTree, parent):
        super(_BaseShapes, self).__init__(spTree, parent)
        self._spTree = spTree
        self._cached_max_shape_id = None

    def __getitem__(self, idx):
        """
        Return shape at *idx* in sequence, e.g. ``shapes[2]``.
        """
        shape_elms = list(self._iter_member_elms())
        try:
            shape_elm = shape_elms[idx]
        except IndexError:
            raise IndexError("shape index out of range")
        return self._shape_factory(shape_elm)

    def __iter__(self):
        """
        Generate a reference to each shape in the collection, in sequence.
        """
        for shape_elm in self._iter_member_elms():
            yield self._shape_factory(shape_elm)

    def __len__(self):
        """
        Return count of shapes in this shape tree. A group shape contributes
        1 to the total, without regard to the number of shapes contained in
        the group.
        """
        shape_elms = list(self._iter_member_elms())
        return len(shape_elms)

    def clone_placeholder(self, placeholder):
        """
        Add a new placeholder shape based on *placeholder*.
        """
        sp = placeholder.element
        ph_type, orient, sz, idx = (sp.ph_type, sp.ph_orient, sp.ph_sz, sp.ph_idx)
        id_ = self._next_shape_id
        name = self._next_ph_name(ph_type, id_, orient)
        self._spTree.add_placeholder(id_, name, ph_type, orient, sz, idx)

    def ph_basename(self, ph_type):
        """
        Return the base name for a placeholder of *ph_type* in this shape
        collection. There is some variance between slide types, for example
        a notes slide uses a different name for the body placeholder, so this
        method can be overriden by subclasses.
        """
        return {
            PP_PLACEHOLDER.BITMAP: "ClipArt Placeholder",
            PP_PLACEHOLDER.BODY: "Text Placeholder",
            PP_PLACEHOLDER.CENTER_TITLE: "Title",
            PP_PLACEHOLDER.CHART: "Chart Placeholder",
            PP_PLACEHOLDER.DATE: "Date Placeholder",
            PP_PLACEHOLDER.FOOTER: "Footer Placeholder",
            PP_PLACEHOLDER.HEADER: "Header Placeholder",
            PP_PLACEHOLDER.MEDIA_CLIP: "Media Placeholder",
            PP_PLACEHOLDER.OBJECT: "Content Placeholder",
            PP_PLACEHOLDER.ORG_CHART: "SmartArt Placeholder",
            PP_PLACEHOLDER.PICTURE: "Picture Placeholder",
            PP_PLACEHOLDER.SLIDE_NUMBER: "Slide Number Placeholder",
            PP_PLACEHOLDER.SUBTITLE: "Subtitle",
            PP_PLACEHOLDER.TABLE: "Table Placeholder",
            PP_PLACEHOLDER.TITLE: "Title",
        }[ph_type]

    @property
    def turbo_add_enabled(self):
        """True if "turbo-add" mode is enabled. Read/Write.

        EXPERIMENTAL: This feature can radically improve performance when
        adding large numbers (hundreds of shapes) to a slide. It works by
        caching the last shape ID used and incrementing that value to assign
        the next shape id. This avoids repeatedly searching all shape ids in
        the slide each time a new ID is required.

        Performance is not noticeably improved for a slide with a relatively
        small number of shapes, but because the search time rises with the
        square of the shape count, this option can be useful for optimizing
        generation of a slide composed of many shapes.

        Shape-id collisions can occur (causing a repair error on load) if
        more than one |Slide| object is used to interact with the same slide
        in the presentation. Note that the |Slides| collection creates a new
        |Slide| object each time a slide is accessed
        (e.g. `slide = prs.slides[0]`, so you must be careful to limit use to
        a single |Slide| object.
        """
        return self._cached_max_shape_id is not None

    @turbo_add_enabled.setter
    def turbo_add_enabled(self, value):
        enable = bool(value)
        self._cached_max_shape_id = self._spTree.max_shape_id if enable else None

    @staticmethod
    def _is_member_elm(shape_elm):
        """
        Return true if *shape_elm* represents a member of this collection,
        False otherwise.
        """
        return True

    def _iter_member_elms(self):
        """
        Generate each child of the ``<p:spTree>`` element that corresponds to
        a shape, in the sequence they appear in the XML.
        """
        for shape_elm in self._spTree.iter_shape_elms():
            if self._is_member_elm(shape_elm):
                yield shape_elm

    def _next_ph_name(self, ph_type, id, orient):
        """
        Next unique placeholder name for placeholder shape of type *ph_type*,
        with id number *id* and orientation *orient*. Usually will be standard
        placeholder root name suffixed with id-1, e.g.
        _next_ph_name(ST_PlaceholderType.TBL, 4, 'horz') ==>
        'Table Placeholder 3'. The number is incremented as necessary to make
        the name unique within the collection. If *orient* is ``'vert'``, the
        placeholder name is prefixed with ``'Vertical '``.
        """
        basename = self.ph_basename(ph_type)

        # prefix rootname with 'Vertical ' if orient is 'vert'
        if orient == ST_Direction.VERT:
            basename = "Vertical %s" % basename

        # increment numpart as necessary to make name unique
        numpart = id - 1
        names = self._spTree.xpath("//p:cNvPr/@name")
        while True:
            name = "%s %d" % (basename, numpart)
            if name not in names:
                break
            numpart += 1

        return name

    @property
    def _next_shape_id(self):
        """Return a unique shape id suitable for use with a new shape.

        The returned id is 1 greater than the maximum shape id used so far.
        In practice, the minimum id is 2 because the spTree element is always
        assigned id="1".
        """
        # ---presence of cached-max-shape-id indicates turbo mode is on---
        if self._cached_max_shape_id is not None:
            self._cached_max_shape_id += 1
            return self._cached_max_shape_id

        return self._spTree.max_shape_id + 1

    def _shape_factory(self, shape_elm):
        """
        Return an instance of the appropriate shape proxy class for
        *shape_elm*.
        """
        return BaseShapeFactory(shape_elm, self)


class _BaseGroupShapes(_BaseShapes):
    """Base class for shape-trees that can add shapes."""

    def __init__(self, grpSp, parent):
        super(_BaseGroupShapes, self).__init__(grpSp, parent)
        self._grpSp = grpSp

    def add_chart(self, chart_type, x, y, cx, cy, chart_data):
        """Add a new chart of *chart_type* to the slide.

        The chart is positioned at (*x*, *y*), has size (*cx*, *cy*), and
        depicts *chart_data*. *chart_type* is one of the :ref:`XlChartType`
        enumeration values. *chart_data* is a |ChartData| object populated
        with the categories and series values for the chart.

        Note that a |GraphicFrame| shape object is returned, not the |Chart|
        object contained in that graphic frame shape. The chart object may be
        accessed using the :attr:`chart` property of the returned
        |GraphicFrame| object.
        """
        rId = self.part.add_chart_part(chart_type, chart_data)
        graphicFrame = self._add_chart_graphicFrame(rId, x, y, cx, cy)
        self._recalculate_extents()
        return self._shape_factory(graphicFrame)

    def add_connector(self, connector_type, begin_x, begin_y, end_x, end_y):
        """Add a newly created connector shape to the end of this shape tree.

        *connector_type* is a member of the :ref:`MsoConnectorType`
        enumeration and the end-point values are specified as EMU values. The
        returned connector is of type *connector_type* and has begin and end
        points as specified.
        """
        cxnSp = self._add_cxnSp(connector_type, begin_x, begin_y, end_x, end_y)
        self._recalculate_extents()
        return self._shape_factory(cxnSp)

    def add_group_shape(self, shapes=[]):
        """Return a |GroupShape| object newly appended to this shape tree.

        The group shape is empty and must be populated with shapes using
        methods on its shape tree, available on its `.shapes` property. The
        position and extents of the group shape are determined by the shapes
        it contains; its position and extents are recalculated each time
        a shape is added to it.
        """
        grpSp = self._element.add_grpSp()
        for shape in shapes:
            grpSp.insert_element_before(shape._element, "p:extLst")
        if shapes:
            grpSp.recalculate_extents()
        return self._shape_factory(grpSp)

    def add_ole_object(
        self, object_file, prog_id, left, top, width=None, height=None, icon_file=None
    ):
        """Return newly-created GraphicFrame shape embedding `object_file`.

        The returned graphic-frame shape contains `object_file` as an embedded OLE
        object. It is displayed as an icon at `left`, `top` with size `width`, `height`.
        `width` and `height` may be omitted when `prog_id` is a member of `PROG_ID`, in
        which case the default icon size is used. This is advised for best appearance
        where applicable because it avoids an icon with a "stretched" appearance.

        `object_file` may either be a str path to the file or a file-like
        object (such as `io.BytesIO`) containing the bytes of the object file.

        `prog_id` can be either a member of `pptx.enum.shapes.PROG_ID` or a str value
        like `"Adobe.Exchange.7"` determined by inspecting the XML generated by
        PowerPoint for an object of the desired type.

        `icon_file` may either be a str path to an image file or a file-like object
        containing the image. The image provided will be displayed in lieu of the OLE
        object; double-clicking on the image opens the object (subject to
        operating-system limitations). The image file can be any supported image file.
        Those produced by PowerPoint itself are generally EMF and can be harvested from
        a PPTX package that embeds such an object. PNG and JPG also work fine.
        """
        graphicFrame = _OleObjectElementCreator.graphicFrame(
            self,
            self._next_shape_id,
            object_file,
            prog_id,
            left,
            top,
            width,
            height,
            icon_file,
        )
        self._spTree.append(graphicFrame)
        self._recalculate_extents()
        return self._shape_factory(graphicFrame)

    def add_picture(self, image_file, left, top, width=None, height=None):
        """Add picture shape displaying image in *image_file*.

        *image_file* can be either a path to a file (a string) or a file-like
        object. The picture is positioned with its top-left corner at (*top*,
        *left*). If *width* and *height* are both |None|, the native size of
        the image is used. If only one of *width* or *height* is used, the
        unspecified dimension is calculated to preserve the aspect ratio of
        the image. If both are specified, the picture is stretched to fit,
        without regard to its native aspect ratio.
        """
        image_part, rId = self.part.get_or_add_image_part(image_file)
        pic = self._add_pic_from_image_part(image_part, rId, left, top, width, height)
        self._recalculate_extents()
        return self._shape_factory(pic)

    def add_shape(self, autoshape_type_id, left, top, width, height):
        """Return new |Shape| object appended to this shape tree.

        *autoshape_type_id* is a member of :ref:`MsoAutoShapeType` e.g.
        ``MSO_SHAPE.RECTANGLE`` specifying the type of shape to be added. The
        remaining arguments specify the new shape's position and size.
        """
        autoshape_type = AutoShapeType(autoshape_type_id)
        sp = self._add_sp(autoshape_type, left, top, width, height)
        self._recalculate_extents()
        return self._shape_factory(sp)

    def add_textbox(self, left, top, width, height):
        """Return newly added text box shape appended to this shape tree.

        The text box is of the specified size, located at the specified
        position on the slide.
        """
        sp = self._add_textbox_sp(left, top, width, height)
        self._recalculate_extents()
        return self._shape_factory(sp)

    def build_freeform(self, start_x=0, start_y=0, scale=1.0):
        """Return |FreeformBuilder| object to specify a freeform shape.

        The optional *start_x* and *start_y* arguments specify the starting
        pen position in local coordinates. They will be rounded to the
        nearest integer before use and each default to zero.

        The optional *scale* argument specifies the size of local coordinates
        proportional to slide coordinates (EMU). If the vertical scale is
        different than the horizontal scale (local coordinate units are
        "rectangular"), a pair of numeric values can be provided as the
        *scale* argument, e.g. `scale=(1.0, 2.0)`. In this case the first
        number is interpreted as the horizontal (X) scale and the second as
        the vertical (Y) scale.

        A convenient method for calculating scale is to divide a |Length|
        object by an equivalent count of local coordinate units, e.g.
        `scale = Inches(1)/1000` for 1000 local units per inch.
        """
        try:
            x_scale, y_scale = scale
        except TypeError:
            x_scale = y_scale = scale

        return FreeformBuilder.new(self, start_x, start_y, x_scale, y_scale)

    def index(self, shape):
        """Return the index of *shape* in this sequence.

        Raises |ValueError| if *shape* is not in the collection.
        """
        shape_elms = list(self._element.iter_shape_elms())
        return shape_elms.index(shape.element)

    def _add_chart_graphicFrame(self, rId, x, y, cx, cy):
        """Return new `p:graphicFrame` element appended to this shape tree.

        The `p:graphicFrame` element has the specified position and size and
        refers to the chart part identified by *rId*.
        """
        shape_id = self._next_shape_id
        name = "Chart %d" % (shape_id - 1)
        graphicFrame = CT_GraphicalObjectFrame.new_chart_graphicFrame(
            shape_id, name, rId, x, y, cx, cy
        )
        self._spTree.append(graphicFrame)
        return graphicFrame

    def _add_cxnSp(self, connector_type, begin_x, begin_y, end_x, end_y):
        """Return a newly-added `p:cxnSp` element as specified.

        The `p:cxnSp` element is for a connector of *connector_type*
        beginning at (*begin_x*, *begin_y*) and extending to
        (*end_x*, *end_y*).
        """
        id_ = self._next_shape_id
        name = "Connector %d" % (id_ - 1)

        flipH, flipV = begin_x > end_x, begin_y > end_y
        x, y = min(begin_x, end_x), min(begin_y, end_y)
        cx, cy = abs(end_x - begin_x), abs(end_y - begin_y)

        return self._element.add_cxnSp(
            id_, name, connector_type, x, y, cx, cy, flipH, flipV
        )

    def _add_pic_from_image_part(self, image_part, rId, x, y, cx, cy):
        """Return a newly appended `p:pic` element as specified.

        The `p:pic` element displays the image in *image_part* with size and
        position specified by *x*, *y*, *cx*, and *cy*. The element is
        appended to the shape tree, causing it to be displayed first in
        z-order on the slide.
        """
        id_ = self._next_shape_id
        scaled_cx, scaled_cy = image_part.scale(cx, cy)
        name = "Picture %d" % (id_ - 1)
        desc = image_part.desc
        pic = self._grpSp.add_pic(id_, name, desc, rId, x, y, scaled_cx, scaled_cy)
        return pic

    def _add_sp(self, autoshape_type, x, y, cx, cy):
        """Return newly-added `p:sp` element as specified.

        `p:sp` element is of *autoshape_type* at position (*x*, *y*) and of
        size (*cx*, *cy*).
        """
        id_ = self._next_shape_id
        name = "%s %d" % (autoshape_type.basename, id_ - 1)
        sp = self._grpSp.add_autoshape(id_, name, autoshape_type.prst, x, y, cx, cy)
        return sp

    def _add_textbox_sp(self, x, y, cx, cy):
        """Return newly-appended textbox `p:sp` element.

        Element has position (*x*, *y*) and size (*cx*, *cy*).
        """
        id_ = self._next_shape_id
        name = "TextBox %d" % (id_ - 1)
        sp = self._spTree.add_textbox(id_, name, x, y, cx, cy)
        return sp

    def _recalculate_extents(self):
        """Adjust position and size to incorporate all contained shapes.

        This would typically be called when a contained shape is added,
        removed, or its position or size updated.
        """
        # ---default behavior is to do nothing, GroupShapes overrides to
        #    produce the distinctive behavior of groups and subgroups.---
        pass


class GroupShapes(_BaseGroupShapes):
    """The sequence of child shapes belonging to a group shape.

    Note that this collection can itself contain a group shape, making this
    part of a recursive, tree data structure (acyclic graph).
    """

    def _recalculate_extents(self):
        """Adjust position and size to incorporate all contained shapes.

        This would typically be called when a contained shape is added,
        removed, or its position or size updated.
        """
        self._grpSp.recalculate_extents()


class SlideShapes(_BaseGroupShapes):
    """Sequence of shapes appearing on a slide.

    The first shape in the sequence is the backmost in z-order and the last
    shape is topmost. Supports indexed access, len(), index(), and iteration.
    """

    def add_movie(
        self,
        movie_file,
        left,
        top,
        width,
        height,
        poster_frame_image=None,
        mime_type=CT.VIDEO,
    ):
        """Return newly added movie shape displaying video in *movie_file*.

        **EXPERIMENTAL.** This method has important limitations:

        * The size must be specified; no auto-scaling such as that provided
          by :meth:`add_picture` is performed.
        * The MIME type of the video file should be specified, e.g.
          'video/mp4'. The provided video file is not interrogated for its
          type. The MIME type `video/unknown` is used by default (and works
          fine in tests as of this writing).
        * A poster frame image must be provided, it cannot be automatically
          extracted from the video file. If no poster frame is provided, the
          default "media loudspeaker" image will be used.

        Return a newly added movie shape to the slide, positioned at (*left*,
        *top*), having size (*width*, *height*), and containing *movie_file*.
        Before the video is started, *poster_frame_image* is displayed as
        a placeholder for the video.
        """
        movie_pic = _MoviePicElementCreator.new_movie_pic(
            self,
            self._next_shape_id,
            movie_file,
            left,
            top,
            width,
            height,
            poster_frame_image,
            mime_type,
        )
        self._spTree.append(movie_pic)
        self._add_video_timing(movie_pic)
        return self._shape_factory(movie_pic)

    def add_table(self, rows, cols, left, top, width, height):
        """
        Add a |GraphicFrame| object containing a table with the specified
        number of *rows* and *cols* and the specified position and size.
        *width* is evenly distributed between the columns of the new table.
        Likewise, *height* is evenly distributed between the rows. Note that
        the ``.table`` property on the returned |GraphicFrame| shape must be
        used to access the enclosed |Table| object.
        """
        graphicFrame = self._add_graphicFrame_containing_table(
            rows, cols, left, top, width, height
        )
        graphic_frame = self._shape_factory(graphicFrame)
        return graphic_frame

    def clone_layout_placeholders(self, slide_layout):
        """
        Add placeholder shapes based on those in *slide_layout*. Z-order of
        placeholders is preserved. Latent placeholders (date, slide number,
        and footer) are not cloned.
        """
        for placeholder in slide_layout.iter_cloneable_placeholders():
            self.clone_placeholder(placeholder)

    @property
    def placeholders(self):
        """
        Instance of |SlidePlaceholders| containing sequence of placeholder
        shapes in this slide.
        """
        return self.parent.placeholders

    @property
    def title(self):
        """
        The title placeholder shape on the slide or |None| if the slide has
        no title placeholder.
        """
        for elm in self._spTree.iter_ph_elms():
            if elm.ph_idx == 0:
                return self._shape_factory(elm)
        return None

    def _add_graphicFrame_containing_table(self, rows, cols, x, y, cx, cy):
        """
        Return a newly added ``<p:graphicFrame>`` element containing a table
        as specified by the parameters.
        """
        _id = self._next_shape_id
        name = "Table %d" % (_id - 1)
        graphicFrame = self._spTree.add_table(_id, name, rows, cols, x, y, cx, cy)
        return graphicFrame

    def _add_video_timing(self, pic):
        """Add a `p:video` element under `p:sld/p:timing`.

        The element will refer to the specified *pic* element by its shape
        id, and cause the video play controls to appear for that video.
        """
        sld = self._spTree.xpath("/p:sld")[0]
        childTnLst = sld.get_or_add_childTnLst()
        childTnLst.add_video(pic.shape_id)

    def _shape_factory(self, shape_elm):
        """
        Return an instance of the appropriate shape proxy class for
        *shape_elm*.
        """
        return SlideShapeFactory(shape_elm, self)


class LayoutShapes(_BaseShapes):
    """
    Sequence of shapes appearing on a slide layout. The first shape in the
    sequence is the backmost in z-order and the last shape is topmost.
    Supports indexed access, len(), index(), and iteration.
    """

    def _shape_factory(self, shape_elm):
        """
        Return an instance of the appropriate shape proxy class for
        *shape_elm*.
        """
        return _LayoutShapeFactory(shape_elm, self)


class MasterShapes(_BaseShapes):
    """
    Sequence of shapes appearing on a slide master. The first shape in the
    sequence is the backmost in z-order and the last shape is topmost.
    Supports indexed access, len(), and iteration.
    """

    def _shape_factory(self, shape_elm):
        """
        Return an instance of the appropriate shape proxy class for
        *shape_elm*.
        """
        return _MasterShapeFactory(shape_elm, self)


class NotesSlideShapes(_BaseShapes):
    """
    Sequence of shapes appearing on a notes slide. The first shape in the
    sequence is the backmost in z-order and the last shape is topmost.
    Supports indexed access, len(), index(), and iteration.
    """

    def ph_basename(self, ph_type):
        """
        Return the base name for a placeholder of *ph_type* in this shape
        collection. A notes slide uses a different name for the body
        placeholder and has some unique placeholder types, so this
        method overrides the default in the base class.
        """
        return {
            PP_PLACEHOLDER.BODY: "Notes Placeholder",
            PP_PLACEHOLDER.DATE: "Date Placeholder",
            PP_PLACEHOLDER.FOOTER: "Footer Placeholder",
            PP_PLACEHOLDER.HEADER: "Header Placeholder",
            PP_PLACEHOLDER.SLIDE_IMAGE: "Slide Image Placeholder",
            PP_PLACEHOLDER.SLIDE_NUMBER: "Slide Number Placeholder",
        }[ph_type]

    def _shape_factory(self, shape_elm):
        """
        Return an instance of the appropriate shape proxy class for
        *shape_elm* appearing on a notes slide.
        """
        return _NotesSlideShapeFactory(shape_elm, self)


class BasePlaceholders(_BaseShapes):
    """
    Base class for placeholder collections that differentiate behaviors for
    a master, layout, and slide. By default, placeholder shapes are
    constructed using |BaseShapeFactory|. Subclasses should override
    :method:`_shape_factory` to use custom placeholder classes.
    """

    @staticmethod
    def _is_member_elm(shape_elm):
        """
        True if *shape_elm* is a placeholder shape, False otherwise.
        """
        return shape_elm.has_ph_elm


class LayoutPlaceholders(BasePlaceholders):
    """
    Sequence of |LayoutPlaceholder| instances representing the placeholder
    shapes on a slide layout.
    """

    def get(self, idx, default=None):
        """
        Return the first placeholder shape with matching *idx* value, or
        *default* if not found.
        """
        for placeholder in self:
            if placeholder.element.ph_idx == idx:
                return placeholder
        return default

    def _shape_factory(self, shape_elm):
        """
        Return an instance of the appropriate shape proxy class for
        *shape_elm*.
        """
        return _LayoutShapeFactory(shape_elm, self)


class MasterPlaceholders(BasePlaceholders):
    """
    Sequence of _MasterPlaceholder instances representing the placeholder
    shapes on a slide master.
    """

    def get(self, ph_type, default=None):
        """
        Return the first placeholder shape with type *ph_type* (e.g. 'body'),
        or *default* if no such placeholder shape is present in the
        collection.
        """
        for placeholder in self:
            if placeholder.ph_type == ph_type:
                return placeholder
        return default

    def _shape_factory(self, shape_elm):
        """
        Return an instance of the appropriate shape proxy class for
        *shape_elm*.
        """
        return _MasterShapeFactory(shape_elm, self)


class NotesSlidePlaceholders(MasterPlaceholders):
    """
    Sequence of placeholder shapes on a notes slide.
    """

    def _shape_factory(self, placeholder_elm):
        """
        Return an instance of the appropriate placeholder proxy class for
        *placeholder_elm*.
        """
        return _NotesSlideShapeFactory(placeholder_elm, self)


class SlidePlaceholders(ParentedElementProxy):
    """
    Collection of placeholder shapes on a slide. Supports iteration,
    :func:`len`, and dictionary-style lookup on the `idx` value of the
    placeholders it contains.
    """

    __slots__ = ()

    def __getitem__(self, idx):
        """
        Access placeholder shape having *idx*. Note that while this looks
        like list access, idx is actually a dictionary key and will raise
        |KeyError| if no placeholder with that idx value is in the
        collection.
        """
        for e in self._element.iter_ph_elms():
            if e.ph_idx == idx:
                return SlideShapeFactory(e, self)
        raise KeyError("no placeholder on this slide with idx == %d" % idx)

    def __iter__(self):
        """
        Generate placeholder shapes in `idx` order.
        """
        ph_elms = sorted(
            [e for e in self._element.iter_ph_elms()], key=lambda e: e.ph_idx
        )
        return (SlideShapeFactory(e, self) for e in ph_elms)

    def __len__(self):
        """
        Return count of placeholder shapes.
        """
        return len(list(self._element.iter_ph_elms()))


def BaseShapeFactory(shape_elm, parent):
    """
    Return an instance of the appropriate shape proxy class for *shape_elm*.
    """
    tag = shape_elm.tag

    if tag == qn("p:pic"):
        videoFiles = shape_elm.xpath("./p:nvPicPr/p:nvPr/a:videoFile")
        if videoFiles:
            return Movie(shape_elm, parent)
        return Picture(shape_elm, parent)

    shape_cls = {
        qn("p:cxnSp"): Connector,
        qn("p:grpSp"): GroupShape,
        qn("p:sp"): Shape,
        qn("p:graphicFrame"): GraphicFrame,
    }.get(tag, BaseShape)

    return shape_cls(shape_elm, parent)


def _LayoutShapeFactory(shape_elm, parent):
    """
    Return an instance of the appropriate shape proxy class for *shape_elm*
    on a slide layout.
    """
    tag_name = shape_elm.tag
    if tag_name == qn("p:sp") and shape_elm.has_ph_elm:
        return LayoutPlaceholder(shape_elm, parent)
    return BaseShapeFactory(shape_elm, parent)


def _MasterShapeFactory(shape_elm, parent):
    """
    Return an instance of the appropriate shape proxy class for *shape_elm*
    on a slide master.
    """
    tag_name = shape_elm.tag
    if tag_name == qn("p:sp") and shape_elm.has_ph_elm:
        return MasterPlaceholder(shape_elm, parent)
    return BaseShapeFactory(shape_elm, parent)


def _NotesSlideShapeFactory(shape_elm, parent):
    """
    Return an instance of the appropriate shape proxy class for *shape_elm*
    on a notes slide.
    """
    tag_name = shape_elm.tag
    if tag_name == qn("p:sp") and shape_elm.has_ph_elm:
        return NotesSlidePlaceholder(shape_elm, parent)
    return BaseShapeFactory(shape_elm, parent)


def _SlidePlaceholderFactory(shape_elm, parent):
    """
    Return a placeholder shape of the appropriate type for *shape_elm*.
    """
    tag = shape_elm.tag
    if tag == qn("p:sp"):
        Constructor = {
            PP_PLACEHOLDER.BITMAP: PicturePlaceholder,
            PP_PLACEHOLDER.CHART: ChartPlaceholder,
            PP_PLACEHOLDER.PICTURE: PicturePlaceholder,
            PP_PLACEHOLDER.TABLE: TablePlaceholder,
        }.get(shape_elm.ph_type, SlidePlaceholder)
    elif tag == qn("p:graphicFrame"):
        Constructor = PlaceholderGraphicFrame
    elif tag == qn("p:pic"):
        Constructor = PlaceholderPicture
    else:
        Constructor = BaseShapeFactory
    return Constructor(shape_elm, parent)


def SlideShapeFactory(shape_elm, parent):
    """
    Return an instance of the appropriate shape proxy class for *shape_elm*
    on a slide.
    """
    if shape_elm.has_ph_elm:
        return _SlidePlaceholderFactory(shape_elm, parent)
    return BaseShapeFactory(shape_elm, parent)


class _MoviePicElementCreator(object):
    """Functional service object for creating a new movie p:pic element.

    It's entire external interface is its :meth:`new_movie_pic` class method
    that returns a new `p:pic` element containing the specified video. This
    class is not intended to be constructed or an instance of it retained by
    the caller; it is a "one-shot" object, really a function wrapped in
    a object such that its helper methods can be organized here.
    """

    def __init__(
        self, shapes, shape_id, movie_file, x, y, cx, cy, poster_frame_file, mime_type
    ):
        super(_MoviePicElementCreator, self).__init__()
        self._shapes = shapes
        self._shape_id = shape_id
        self._movie_file = movie_file
        self._x, self._y, self._cx, self._cy = x, y, cx, cy
        self._poster_frame_file = poster_frame_file
        self._mime_type = mime_type

    @classmethod
    def new_movie_pic(
        cls, shapes, shape_id, movie_file, x, y, cx, cy, poster_frame_image, mime_type
    ):
        """Return a new `p:pic` element containing video in *movie_file*.

        If *mime_type* is None, 'video/unknown' is used. If
        *poster_frame_file* is None, the default "media loudspeaker" image is
        used.
        """
        return cls(
            shapes, shape_id, movie_file, x, y, cx, cy, poster_frame_image, mime_type
        )._pic
        return

    @property
    def _media_rId(self):
        """Return the rId of RT.MEDIA relationship to video part.

        For historical reasons, there are two relationships to the same part;
        one is the video rId and the other is the media rId.
        """
        return self._video_part_rIds[0]

    @lazyproperty
    def _pic(self):
        """Return the new `p:pic` element referencing the video."""
        return CT_Picture.new_video_pic(
            self._shape_id,
            self._shape_name,
            self._video_rId,
            self._media_rId,
            self._poster_frame_rId,
            self._x,
            self._y,
            self._cx,
            self._cy,
        )

    @lazyproperty
    def _poster_frame_image_file(self):
        """Return the image file for video placeholder image.

        If no poster frame file is provided, the default "media loudspeaker"
        image is used.
        """
        poster_frame_file = self._poster_frame_file
        if poster_frame_file is None:
            return BytesIO(SPEAKER_IMAGE_BYTES)
        return poster_frame_file

    @lazyproperty
    def _poster_frame_rId(self):
        """Return the rId of relationship to poster frame image.

        The poster frame is the image used to represent the video before it's
        played.
        """
        _, poster_frame_rId = self._slide_part.get_or_add_image_part(
            self._poster_frame_image_file
        )
        return poster_frame_rId

    @property
    def _shape_name(self):
        """Return the appropriate shape name for the p:pic shape.

        A movie shape is named with the base filename of the video.
        """
        return self._video.filename

    @property
    def _slide_part(self):
        """Return SlidePart object for slide containing this movie."""
        return self._shapes.part

    @lazyproperty
    def _video(self):
        """Return a |Video| object containing the movie file."""
        return Video.from_path_or_file_like(self._movie_file, self._mime_type)

    @lazyproperty
    def _video_part_rIds(self):
        """Return the rIds for relationships to media part for video.

        This is where the media part and its relationships to the slide are
        actually created.
        """
        media_rId, video_rId = self._slide_part.get_or_add_video_media_part(self._video)
        return media_rId, video_rId

    @property
    def _video_rId(self):
        """Return the rId of RT.VIDEO relationship to video part.

        For historical reasons, there are two relationships to the same part;
        one is the video rId and the other is the media rId.
        """
        return self._video_part_rIds[1]


class _OleObjectElementCreator(object):
    """Functional service object for creating a new OLE-object p:graphicFrame element.

    It's entire external interface is its :meth:`graphicFrame` class method that returns
    a new `p:graphicFrame` element containing the specified embedded OLE-object shape.
    This class is not intended to be constructed or an instance of it retained by the
    caller; it is a "one-shot" object, really a function wrapped in a object such that
    its helper methods can be organized here.
    """

    def __init__(
        self, shapes, shape_id, ole_object_file, prog_id, x, y, cx, cy, icon_file
    ):
        self._shapes = shapes
        self._shape_id = shape_id
        self._ole_object_file = ole_object_file
        self._prog_id_arg = prog_id
        self._x = x
        self._y = y
        self._cx_arg = cx
        self._cy_arg = cy
        self._icon_file_arg = icon_file

    @classmethod
    def graphicFrame(
        cls, shapes, shape_id, ole_object_file, prog_id, x, y, cx, cy, icon_file
    ):
        """Return new `p:graphicFrame` element containing embedded `ole_object_file`."""
        return cls(
            shapes, shape_id, ole_object_file, prog_id, x, y, cx, cy, icon_file
        )._graphicFrame

    @lazyproperty
    def _graphicFrame(self):
        """Newly-created `p:graphicFrame` element referencing embedded OLE-object."""
        return CT_GraphicalObjectFrame.new_ole_object_graphicFrame(
            self._shape_id,
            self._shape_name,
            self._ole_object_rId,
            self._progId,
            self._icon_rId,
            self._x,
            self._y,
            self._cx,
            self._cy,
        )

    @lazyproperty
    def _cx(self):
        """Emu object specifying width of "show-as-icon" image for OLE shape."""
        # --- a user-specified width overrides any default ---
        if self._cx_arg is not None:
            return self._cx_arg

        # --- the default width is specified by the PROG_ID member if prog_id is one,
        # --- otherwise it gets the default icon width.
        return (
            Emu(self._prog_id_arg.width)
            if self._prog_id_arg in PROG_ID
            else Emu(965200)
        )

    @lazyproperty
    def _cy(self):
        """Emu object specifying height of "show-as-icon" image for OLE shape."""
        # --- a user-specified width overrides any default ---
        if self._cy_arg is not None:
            return self._cy_arg

        # --- the default height is specified by the PROG_ID member if prog_id is one,
        # --- otherwise it gets the default icon height.
        return (
            Emu(self._prog_id_arg.height)
            if self._prog_id_arg in PROG_ID
            else Emu(609600)
        )

    @lazyproperty
    def _icon_image_file(self):
        """Reference to image file containing icon to show in lieu of this object.

        This can be either a str path or a file-like object (io.BytesIO typically).
        """
        # --- a user-specified icon overrides any default ---
        if self._icon_file_arg is not None:
            return self._icon_file_arg

        # --- A prog_id belonging to PROG_ID gets its icon filename from there. A
        # --- user-specified (str) prog_id gets the default icon.
        icon_filename = (
            self._prog_id_arg.icon_filename
            if self._prog_id_arg in PROG_ID
            else "generic-icon.emf"
        )

        _thisdir = os.path.split(__file__)[0]
        return os.path.abspath(os.path.join(_thisdir, "..", "templates", icon_filename))

    @lazyproperty
    def _icon_rId(self):
        """str rId like "rId7" of rel to icon (image) representing OLE-object part."""
        _, rId = self._slide_part.get_or_add_image_part(self._icon_image_file)
        return rId

    @lazyproperty
    def _ole_object_rId(self):
        """str rId like "rId6" of relationship to embedded ole_object part.

        This is where the ole_object part and its relationship to the slide are actually
        created.
        """
        return self._slide_part.add_embedded_ole_object_part(
            self._prog_id_arg, self._ole_object_file
        )

    @lazyproperty
    def _progId(self):
        """str like "Excel.Sheet.12" identifying program used to open object.

        This value appears in the `progId` attribute of the `p:oleObj` element for the
        object.
        """
        prog_id_arg = self._prog_id_arg

        # --- member of PROG_ID enumeration knows its progId keyphrase, otherwise caller
        # --- has specified it explicitly (as str)
        return prog_id_arg.progId if prog_id_arg in PROG_ID else prog_id_arg

    @lazyproperty
    def _shape_name(self):
        """str name like "Object 1" for the embedded ole_object shape.

        The name is formed from the prefix "Object " and the shape-id decremented by 1.
        """
        return "Object %d" % (self._shape_id - 1)

    @lazyproperty
    def _slide_part(self):
        """SlidePart object for this slide."""
        return self._shapes.part



# encoding: utf-8

"""
|ChartFormat| and related objects. |ChartFormat| acts as proxy for the `spPr`
element, which provides visual shape properties such as line and fill for
chart elements.
"""

from __future__ import absolute_import, division, print_function, unicode_literals

from .fill import FillFormat
from .line import LineFormat
from ..shared import ElementProxy
from ..util import lazyproperty


class ChartFormat(ElementProxy):
    """
    The |ChartFormat| object provides access to visual shape properties for
    chart elements like |Axis|, |Series|, and |MajorGridlines|. It has two
    properties, :attr:`fill` and :attr:`line`, which return a |FillFormat|
    and |LineFormat| object respectively. The |ChartFormat| object is
    provided by the :attr:`format` property on the target axis, series, etc.
    """

    __slots__ = ("_fill", "_line")

    @lazyproperty
    def fill(self):
        """
        |FillFormat| instance for this object, providing access to fill
        properties such as fill color.
        """
        spPr = self._element.get_or_add_spPr()
        return FillFormat.from_fill_parent(spPr)

    @lazyproperty
    def line(self):
        """
        The |LineFormat| object providing access to the visual properties of
        this object, such as line color and line style.
        """
        spPr = self._element.get_or_add_spPr()
        return LineFormat(spPr)



# encoding: utf-8

"""Enumerations used by DrawingML objects."""

from __future__ import absolute_import, division, print_function, unicode_literals

from .base import (
    alias,
    Enumeration,
    EnumMember,
    ReturnValueOnlyEnumMember,
    XmlEnumeration,
    XmlMappedEnumMember,
)


class MSO_COLOR_TYPE(Enumeration):
    """
    Specifies the color specification scheme

    Example::

        from pptx.enum.dml import MSO_COLOR_TYPE

        assert shape.fill.fore_color.type == MSO_COLOR_TYPE.SCHEME
    """

    __ms_name__ = "MsoColorType"

    __url__ = (
        "http://msdn.microsoft.com/en-us/library/office/ff864912(v=office.15" ").aspx"
    )

    __members__ = (
        EnumMember("RGB", 1, "Color is specified by an |RGBColor| value"),
        EnumMember("SCHEME", 2, "Color is one of the preset theme colors"),
        EnumMember(
            "HSL",
            101,
            """
            Color is specified using Hue, Saturation, and Luminosity values
            """,
        ),
        EnumMember(
            "PRESET",
            102,
            """
            Color is specified using a named built-in color
            """,
        ),
        EnumMember(
            "SCRGB",
            103,
            """
            Color is an scRGB color, a wide color gamut RGB color space
            """,
        ),
        EnumMember(
            "SYSTEM",
            104,
            """
            Color is one specified by the operating system, such as the
            window background color.
            """,
        ),
    )


@alias("MSO_FILL")
class MSO_FILL_TYPE(Enumeration):
    """
    Specifies the type of bitmap used for the fill of a shape.

    Alias: ``MSO_FILL``

    Example::

        from pptx.enum.dml import MSO_FILL

        assert shape.fill.type == MSO_FILL.SOLID
    """

    __ms_name__ = "MsoFillType"

    __url__ = "http://msdn.microsoft.com/EN-US/library/office/ff861408.aspx"

    __members__ = (
        EnumMember(
            "BACKGROUND",
            5,
            """
            The shape is transparent, such that whatever is behind the shape
            shows through. Often this is the slide background, but if
            a visible shape is behind, that will show through.
            """,
        ),
        EnumMember("GRADIENT", 3, "Shape is filled with a gradient"),
        EnumMember(
            "GROUP",
            101,
            "Shape is part of a group and should inherit the "
            "fill properties of the group.",
        ),
        EnumMember("PATTERNED", 2, "Shape is filled with a pattern"),
        EnumMember("PICTURE", 6, "Shape is filled with a bitmapped image"),
        EnumMember("SOLID", 1, "Shape is filled with a solid color"),
        EnumMember("TEXTURED", 4, "Shape is filled with a texture"),
    )


@alias("MSO_LINE")
class MSO_LINE_DASH_STYLE(XmlEnumeration):
    """Specifies the dash style for a line.

    Alias: ``MSO_LINE``

    Example::

        from pptx.enum.dml import MSO_LINE

        shape.line.dash_style = MSO_LINE.DASH_DOT_DOT
    """

    __ms_name__ = "MsoLineDashStyle"

    __url__ = (
        "https://msdn.microsoft.com/en-us/vba/office-shared-vba/articles/mso"
        "linedashstyle-enumeration-office"
    )

    __members__ = (
        XmlMappedEnumMember("DASH", 4, "dash", "Line consists of dashes only."),
        XmlMappedEnumMember("DASH_DOT", 5, "dashDot", "Line is a dash-dot pattern."),
        XmlMappedEnumMember(
            "DASH_DOT_DOT", 6, "lgDashDotDot", "Line is a dash-dot-dot patte" "rn."
        ),
        XmlMappedEnumMember("LONG_DASH", 7, "lgDash", "Line consists of long dashes."),
        XmlMappedEnumMember(
            "LONG_DASH_DOT", 8, "lgDashDot", "Line is a long dash-dot patter" "n."
        ),
        XmlMappedEnumMember("ROUND_DOT", 3, "dot", "Line is made up of round dots."),
        XmlMappedEnumMember("SOLID", 1, "solid", "Line is solid."),
        XmlMappedEnumMember(
            "SQUARE_DOT", 2, "sysDash", "Line is made up of square dots."
        ),
        ReturnValueOnlyEnumMember("DASH_STYLE_MIXED", -2, "Not supported."),
    )


@alias("MSO_PATTERN")
class MSO_PATTERN_TYPE(XmlEnumeration):
    """Specifies the fill pattern used in a shape.

    Alias: ``MSO_PATTERN``

    Example::

        from pptx.enum.dml import MSO_PATTERN

        fill = shape.fill
        fill.patterned()
        fill.pattern = MSO_PATTERN.WAVE
    """

    __ms_name__ = "MsoPatternType"

    __url__ = (
        "https://msdn.microsoft.com/VBA/Office-Shared-VBA/articles/msopatter"
        "ntype-enumeration-office"
    )

    __members__ = (
        XmlMappedEnumMember("CROSS", 51, "cross", "Cross"),
        XmlMappedEnumMember(
            "DARK_DOWNWARD_DIAGONAL", 15, "dkDnDiag", "Dark Downward Diagona" "l"
        ),
        XmlMappedEnumMember("DARK_HORIZONTAL", 13, "dkHorz", "Dark Horizontal"),
        XmlMappedEnumMember(
            "DARK_UPWARD_DIAGONAL", 16, "dkUpDiag", "Dark Upward Diagonal"
        ),
        XmlMappedEnumMember("DARK_VERTICAL", 14, "dkVert", "Dark Vertical"),
        XmlMappedEnumMember(
            "DASHED_DOWNWARD_DIAGONAL", 28, "dashDnDiag", "Dashed Downward D" "iagonal"
        ),
        XmlMappedEnumMember("DASHED_HORIZONTAL", 32, "dashHorz", "Dashed Horizontal"),
        XmlMappedEnumMember(
            "DASHED_UPWARD_DIAGONAL", 27, "dashUpDiag", "Dashed Upward Diago" "nal"
        ),
        XmlMappedEnumMember("DASHED_VERTICAL", 31, "dashVert", "Dashed Vertical"),
        XmlMappedEnumMember("DIAGONAL_BRICK", 40, "diagBrick", "Diagonal Brick"),
        XmlMappedEnumMember("DIAGONAL_CROSS", 54, "diagCross", "Diagonal Cross"),
        XmlMappedEnumMember("DIVOT", 46, "divot", "Pattern Divot"),
        XmlMappedEnumMember("DOTTED_DIAMOND", 24, "dotDmnd", "Dotted Diamond"),
        XmlMappedEnumMember("DOTTED_GRID", 45, "dotGrid", "Dotted Grid"),
        XmlMappedEnumMember("DOWNWARD_DIAGONAL", 52, "dnDiag", "Downward Diagonal"),
        XmlMappedEnumMember("HORIZONTAL", 49, "horz", "Horizontal"),
        XmlMappedEnumMember("HORIZONTAL_BRICK", 35, "horzBrick", "Horizontal Brick"),
        XmlMappedEnumMember(
            "LARGE_CHECKER_BOARD", 36, "lgCheck", "Large Checker Board"
        ),
        XmlMappedEnumMember("LARGE_CONFETTI", 33, "lgConfetti", "Large Confetti"),
        XmlMappedEnumMember("LARGE_GRID", 34, "lgGrid", "Large Grid"),
        XmlMappedEnumMember(
            "LIGHT_DOWNWARD_DIAGONAL", 21, "ltDnDiag", "Light Downward Diago" "nal"
        ),
        XmlMappedEnumMember("LIGHT_HORIZONTAL", 19, "ltHorz", "Light Horizontal"),
        XmlMappedEnumMember(
            "LIGHT_UPWARD_DIAGONAL", 22, "ltUpDiag", "Light Upward Diagonal"
        ),
        XmlMappedEnumMember("LIGHT_VERTICAL", 20, "ltVert", "Light Vertical"),
        XmlMappedEnumMember("NARROW_HORIZONTAL", 30, "narHorz", "Narrow Horizontal"),
        XmlMappedEnumMember("NARROW_VERTICAL", 29, "narVert", "Narrow Vertical"),
        XmlMappedEnumMember("OUTLINED_DIAMOND", 41, "openDmnd", "Outlined Diamond"),
        XmlMappedEnumMember("PERCENT_10", 2, "pct10", "10% of the foreground color."),
        XmlMappedEnumMember("PERCENT_20", 3, "pct20", "20% of the foreground color."),
        XmlMappedEnumMember("PERCENT_25", 4, "pct25", "25% of the foreground color."),
        XmlMappedEnumMember("PERCENT_30", 5, "pct30", "30% of the foreground color."),
        XmlMappedEnumMember("PERCENT_40", 6, "pct40", "40% of the foreground color."),
        XmlMappedEnumMember("PERCENT_5", 1, "pct5", "5% of the foreground color."),
        XmlMappedEnumMember("PERCENT_50", 7, "pct50", "50% of the foreground color."),
        XmlMappedEnumMember("PERCENT_60", 8, "pct60", "60% of the foreground color."),
        XmlMappedEnumMember("PERCENT_70", 9, "pct70", "70% of the foreground color."),
        XmlMappedEnumMember("PERCENT_75", 10, "pct75", "75% of the foreground color."),
        XmlMappedEnumMember("PERCENT_80", 11, "pct80", "80% of the foreground color."),
        XmlMappedEnumMember("PERCENT_90", 12, "pct90", "90% of the foreground color."),
        XmlMappedEnumMember("PLAID", 42, "plaid", "Plaid"),
        XmlMappedEnumMember("SHINGLE", 47, "shingle", "Shingle"),
        XmlMappedEnumMember(
            "SMALL_CHECKER_BOARD", 17, "smCheck", "Small Checker Board"
        ),
        XmlMappedEnumMember("SMALL_CONFETTI", 37, "smConfetti", "Small Confetti"),
        XmlMappedEnumMember("SMALL_GRID", 23, "smGrid", "Small Grid"),
        XmlMappedEnumMember("SOLID_DIAMOND", 39, "solidDmnd", "Solid Diamond"),
        XmlMappedEnumMember("SPHERE", 43, "sphere", "Sphere"),
        XmlMappedEnumMember("TRELLIS", 18, "trellis", "Trellis"),
        XmlMappedEnumMember("UPWARD_DIAGONAL", 53, "upDiag", "Upward Diagonal"),
        XmlMappedEnumMember("VERTICAL", 50, "vert", "Vertical"),
        XmlMappedEnumMember("WAVE", 48, "wave", "Wave"),
        XmlMappedEnumMember("WEAVE", 44, "weave", "Weave"),
        XmlMappedEnumMember(
            "WIDE_DOWNWARD_DIAGONAL", 25, "wdDnDiag", "Wide Downward Diagona" "l"
        ),
        XmlMappedEnumMember(
            "WIDE_UPWARD_DIAGONAL", 26, "wdUpDiag", "Wide Upward Diagonal"
        ),
        XmlMappedEnumMember("ZIG_ZAG", 38, "zigZag", "Zig Zag"),
        ReturnValueOnlyEnumMember("MIXED", -2, "Mixed pattern."),
    )


@alias("MSO_THEME_COLOR")
class MSO_THEME_COLOR_INDEX(XmlEnumeration):
    """
    Indicates the Office theme color, one of those shown in the color gallery
    on the formatting ribbon.

    Alias: ``MSO_THEME_COLOR``

    Example::

        from pptx.enum.dml import MSO_THEME_COLOR

        shape.fill.solid()
        shape.fill.fore_color.theme_color = MSO_THEME_COLOR.ACCENT_1
    """

    __ms_name__ = "MsoThemeColorIndex"

    __url__ = (
        "http://msdn.microsoft.com/en-us/library/office/ff860782(v=office.15" ").aspx"
    )

    __members__ = (
        EnumMember("NOT_THEME_COLOR", 0, "Indicates the color is not a theme color."),
        XmlMappedEnumMember(
            "ACCENT_1", 5, "accent1", "Specifies the Accent 1 theme color."
        ),
        XmlMappedEnumMember(
            "ACCENT_2", 6, "accent2", "Specifies the Accent 2 theme color."
        ),
        XmlMappedEnumMember(
            "ACCENT_3", 7, "accent3", "Specifies the Accent 3 theme color."
        ),
        XmlMappedEnumMember(
            "ACCENT_4", 8, "accent4", "Specifies the Accent 4 theme color."
        ),
        XmlMappedEnumMember(
            "ACCENT_5", 9, "accent5", "Specifies the Accent 5 theme color."
        ),
        XmlMappedEnumMember(
            "ACCENT_6", 10, "accent6", "Specifies the Accent 6 theme color."
        ),
        XmlMappedEnumMember(
            "BACKGROUND_1", 14, "bg1", "Specifies the Background 1 theme " "color."
        ),
        XmlMappedEnumMember(
            "BACKGROUND_2", 16, "bg2", "Specifies the Background 2 theme " "color."
        ),
        XmlMappedEnumMember("DARK_1", 1, "dk1", "Specifies the Dark 1 theme color."),
        XmlMappedEnumMember("DARK_2", 3, "dk2", "Specifies the Dark 2 theme color."),
        XmlMappedEnumMember(
            "FOLLOWED_HYPERLINK",
            12,
            "folHlink",
            "Specifies the theme color" " for a clicked hyperlink.",
        ),
        XmlMappedEnumMember(
            "HYPERLINK", 11, "hlink", "Specifies the theme color for a hyper" "link."
        ),
        XmlMappedEnumMember("LIGHT_1", 2, "lt1", "Specifies the Light 1 theme color."),
        XmlMappedEnumMember("LIGHT_2", 4, "lt2", "Specifies the Light 2 theme color."),
        XmlMappedEnumMember("TEXT_1", 13, "tx1", "Specifies the Text 1 theme color."),
        XmlMappedEnumMember("TEXT_2", 15, "tx2", "Specifies the Text 2 theme color."),
        ReturnValueOnlyEnumMember(
            "MIXED",
            -2,
            "Indicates multiple theme colors are used, such as " "in a group shape.",
        ),
    )



# encoding: utf-8

"""
lxml custom element classes for DrawingML-related XML elements.
"""

from __future__ import absolute_import

from ...enum.dml import MSO_THEME_COLOR
from ..simpletypes import ST_HexColorRGB, ST_Percentage
from ..xmlchemy import (
    BaseOxmlElement,
    Choice,
    RequiredAttribute,
    ZeroOrOne,
    ZeroOrOneChoice,
)


class _BaseColorElement(BaseOxmlElement):
    """
    Base class for <a:srgbClr> and <a:schemeClr> elements.
    """

    lumMod = ZeroOrOne("a:lumMod")
    lumOff = ZeroOrOne("a:lumOff")

    def add_lumMod(self, value):
        """
        Return a newly added <a:lumMod> child element.
        """
        lumMod = self._add_lumMod()
        lumMod.val = value
        return lumMod

    def add_lumOff(self, value):
        """
        Return a newly added <a:lumOff> child element.
        """
        lumOff = self._add_lumOff()
        lumOff.val = value
        return lumOff

    def clear_lum(self):
        """
        Return self after removing any <a:lumMod> and <a:lumOff> child
        elements.
        """
        self._remove_lumMod()
        self._remove_lumOff()
        return self


class CT_Color(BaseOxmlElement):
    """Custom element class for `a:fgClr`, `a:bgClr` and perhaps others."""

    eg_colorChoice = ZeroOrOneChoice(
        (
            Choice("a:scrgbClr"),
            Choice("a:srgbClr"),
            Choice("a:hslClr"),
            Choice("a:sysClr"),
            Choice("a:schemeClr"),
            Choice("a:prstClr"),
        ),
        successors=(),
    )


class CT_HslColor(_BaseColorElement):
    """
    Custom element class for <a:hslClr> element.
    """


class CT_Percentage(BaseOxmlElement):
    """
    Custom element class for <a:lumMod> and <a:lumOff> elements.
    """

    val = RequiredAttribute("val", ST_Percentage)


class CT_PresetColor(_BaseColorElement):
    """
    Custom element class for <a:prstClr> element.
    """


class CT_SchemeColor(_BaseColorElement):
    """
    Custom element class for <a:schemeClr> element.
    """

    val = RequiredAttribute("val", MSO_THEME_COLOR)


class CT_ScRgbColor(_BaseColorElement):
    """
    Custom element class for <a:scrgbClr> element.
    """


class CT_SRgbColor(_BaseColorElement):
    """
    Custom element class for <a:srgbClr> element.
    """

    val = RequiredAttribute("val", ST_HexColorRGB)


class CT_SystemColor(_BaseColorElement):
    """
    Custom element class for <a:sysClr> element.
    """



# encoding: utf-8

"""
Provides Python 2/3 compatibility objects
"""

from __future__ import absolute_import, division, print_function, unicode_literals

import sys

import collections

try:
    Sequence = collections.abc.Sequence
except AttributeError:
    Sequence = collections.Sequence

if sys.version_info >= (3, 0):
    from .python3 import (  # noqa
        BytesIO,
        is_integer,
        is_string,
        is_unicode,
        to_unicode,
        Unicode,
    )
else:
    from .python2 import (  # noqa
        BytesIO,
        is_integer,
        is_string,
        is_unicode,
        to_unicode,
        Unicode,
    )



# encoding: utf-8

"""Enumerations used by shapes and related objects."""

from pptx.enum.base import (
    alias,
    Enumeration,
    EnumMember,
    ReturnValueOnlyEnumMember,
    XmlEnumeration,
    XmlMappedEnumMember,
)
from pptx.util import lazyproperty


@alias("MSO_SHAPE")
class MSO_AUTO_SHAPE_TYPE(XmlEnumeration):
    """
    Specifies a type of AutoShape, e.g. DOWN_ARROW

    Alias: ``MSO_SHAPE``

    Example::

        from pptx.enum.shapes import MSO_SHAPE
        from pptx.util import Inches

        left = top = width = height = Inches(1.0)
        slide.shapes.add_shape(
            MSO_SHAPE.ROUNDED_RECTANGLE, left, top, width, height
        )
    """

    __ms_name__ = "MsoAutoShapeType"

    __url__ = (
        "http://msdn.microsoft.com/en-us/library/office/ff862770(v=office.15" ").aspx"
    )

    __members__ = (
        XmlMappedEnumMember(
            "ACTION_BUTTON_BACK_OR_PREVIOUS",
            129,
            "actionButtonBackPrevious",
            "Back or Previous button. Supports " "mouse-click and mouse-over actions",
        ),
        XmlMappedEnumMember(
            "ACTION_BUTTON_BEGINNING",
            131,
            "actionButtonBeginning",
            "Beginning button. Supports mouse-click and mouse-over actions",
        ),
        XmlMappedEnumMember(
            "ACTION_BUTTON_CUSTOM",
            125,
            "actionButtonBlank",
            "Button with no default picture or text. Supports mouse-click an"
            "d mouse-over actions",
        ),
        XmlMappedEnumMember(
            "ACTION_BUTTON_DOCUMENT",
            134,
            "actionButtonDocument",
            "Document button. Supports mouse-click and mouse-over actions",
        ),
        XmlMappedEnumMember(
            "ACTION_BUTTON_END",
            132,
            "actionButtonEnd",
            "End button. Supports mouse-click and mouse-over actions",
        ),
        XmlMappedEnumMember(
            "ACTION_BUTTON_FORWARD_OR_NEXT",
            130,
            "actionButtonForwardNext",
            "Forward or Next button. Supports mouse-click and mouse-over act" "ions",
        ),
        XmlMappedEnumMember(
            "ACTION_BUTTON_HELP",
            127,
            "actionButtonHelp",
            "Help button. Supports mouse-click and mouse-over actions",
        ),
        XmlMappedEnumMember(
            "ACTION_BUTTON_HOME",
            126,
            "actionButtonHome",
            "Home button. Supports mouse-click and mouse-over actions",
        ),
        XmlMappedEnumMember(
            "ACTION_BUTTON_INFORMATION",
            128,
            "actionButtonInformation",
            "Information button. Supports mouse-click and mouse-over actions",
        ),
        XmlMappedEnumMember(
            "ACTION_BUTTON_MOVIE",
            136,
            "actionButtonMovie",
            "Movie button. Supports mouse-click and mouse-over actions",
        ),
        XmlMappedEnumMember(
            "ACTION_BUTTON_RETURN",
            133,
            "actionButtonReturn",
            "Return button. Supports mouse-click and mouse-over actions",
        ),
        XmlMappedEnumMember(
            "ACTION_BUTTON_SOUND",
            135,
            "actionButtonSound",
            "Sound button. Supports mouse-click and mouse-over actions",
        ),
        XmlMappedEnumMember("ARC", 25, "arc", "Arc"),
        XmlMappedEnumMember(
            "BALLOON", 137, "wedgeRoundRectCallout", "Rounded Rectangular Callout"
        ),
        XmlMappedEnumMember(
            "BENT_ARROW",
            41,
            "bentArrow",
            "Block arrow that follows a curved 90-degree angle",
        ),
        XmlMappedEnumMember(
            "BENT_UP_ARROW",
            44,
            "bentUpArrow",
            "Block arrow that follows a sharp 90-degree angle. Points up by " "default",
        ),
        XmlMappedEnumMember("BEVEL", 15, "bevel", "Bevel"),
        XmlMappedEnumMember("BLOCK_ARC", 20, "blockArc", "Block arc"),
        XmlMappedEnumMember("CAN", 13, "can", "Can"),
        XmlMappedEnumMember("CHART_PLUS", 182, "chartPlus", "Chart Plus"),
        XmlMappedEnumMember("CHART_STAR", 181, "chartStar", "Chart Star"),
        XmlMappedEnumMember("CHART_X", 180, "chartX", "Chart X"),
        XmlMappedEnumMember("CHEVRON", 52, "chevron", "Chevron"),
        XmlMappedEnumMember("CHORD", 161, "chord", "Geometric chord shape"),
        XmlMappedEnumMember(
            "CIRCULAR_ARROW",
            60,
            "circularArrow",
            "Block arrow that follows a curved 180-degree angle",
        ),
        XmlMappedEnumMember("CLOUD", 179, "cloud", "Cloud"),
        XmlMappedEnumMember("CLOUD_CALLOUT", 108, "cloudCallout", "Cloud callout"),
        XmlMappedEnumMember("CORNER", 162, "corner", "Corner"),
        XmlMappedEnumMember("CORNER_TABS", 169, "cornerTabs", "Corner Tabs"),
        XmlMappedEnumMember("CROSS", 11, "plus", "Cross"),
        XmlMappedEnumMember("CUBE", 14, "cube", "Cube"),
        XmlMappedEnumMember(
            "CURVED_DOWN_ARROW", 48, "curvedDownArrow", "Block arrow that curves down"
        ),
        XmlMappedEnumMember(
            "CURVED_DOWN_RIBBON", 100, "ellipseRibbon", "Ribbon banner that curves down"
        ),
        XmlMappedEnumMember(
            "CURVED_LEFT_ARROW", 46, "curvedLeftArrow", "Block arrow that curves left"
        ),
        XmlMappedEnumMember(
            "CURVED_RIGHT_ARROW",
            45,
            "curvedRightArrow",
            "Block arrow that curves right",
        ),
        XmlMappedEnumMember(
            "CURVED_UP_ARROW", 47, "curvedUpArrow", "Block arrow that curves up"
        ),
        XmlMappedEnumMember(
            "CURVED_UP_RIBBON", 99, "ellipseRibbon2", "Ribbon banner that curves up"
        ),
        XmlMappedEnumMember("DECAGON", 144, "decagon", "Decagon"),
        XmlMappedEnumMember("DIAGONAL_STRIPE", 141, "diagStripe", "Diagonal Stripe"),
        XmlMappedEnumMember("DIAMOND", 4, "diamond", "Diamond"),
        XmlMappedEnumMember("DODECAGON", 146, "dodecagon", "Dodecagon"),
        XmlMappedEnumMember("DONUT", 18, "donut", "Donut"),
        XmlMappedEnumMember("DOUBLE_BRACE", 27, "bracePair", "Double brace"),
        XmlMappedEnumMember("DOUBLE_BRACKET", 26, "bracketPair", "Double bracket"),
        XmlMappedEnumMember("DOUBLE_WAVE", 104, "doubleWave", "Double wave"),
        XmlMappedEnumMember(
            "DOWN_ARROW", 36, "downArrow", "Block arrow that points down"
        ),
        XmlMappedEnumMember(
            "DOWN_ARROW_CALLOUT",
            56,
            "downArrowCallout",
            "Callout with arrow that points down",
        ),
        XmlMappedEnumMember(
            "DOWN_RIBBON",
            98,
            "ribbon",
            "Ribbon banner with center area below ribbon ends",
        ),
        XmlMappedEnumMember("EXPLOSION1", 89, "irregularSeal1", "Explosion"),
        XmlMappedEnumMember("EXPLOSION2", 90, "irregularSeal2", "Explosion"),
        XmlMappedEnumMember(
            "FLOWCHART_ALTERNATE_PROCESS",
            62,
            "flowChartAlternateProcess",
            "Alternate process flowchart symbol",
        ),
        XmlMappedEnumMember(
            "FLOWCHART_CARD", 75, "flowChartPunchedCard", "Card flowchart symbol"
        ),
        XmlMappedEnumMember(
            "FLOWCHART_COLLATE", 79, "flowChartCollate", "Collate flowchart symbol"
        ),
        XmlMappedEnumMember(
            "FLOWCHART_CONNECTOR",
            73,
            "flowChartConnector",
            "Connector flowchart symbol",
        ),
        XmlMappedEnumMember(
            "FLOWCHART_DATA", 64, "flowChartInputOutput", "Data flowchart symbol"
        ),
        XmlMappedEnumMember(
            "FLOWCHART_DECISION", 63, "flowChartDecision", "Decision flowchart symbol"
        ),
        XmlMappedEnumMember(
            "FLOWCHART_DELAY", 84, "flowChartDelay", "Delay flowchart symbol"
        ),
        XmlMappedEnumMember(
            "FLOWCHART_DIRECT_ACCESS_STORAGE",
            87,
            "flowChartMagneticDrum",
            "Direct access storage flowchart symbol",
        ),
        XmlMappedEnumMember(
            "FLOWCHART_DISPLAY", 88, "flowChartDisplay", "Display flowchart symbol"
        ),
        XmlMappedEnumMember(
            "FLOWCHART_DOCUMENT", 67, "flowChartDocument", "Document flowchart symbol"
        ),
        XmlMappedEnumMember(
            "FLOWCHART_EXTRACT", 81, "flowChartExtract", "Extract flowchart symbol"
        ),
        XmlMappedEnumMember(
            "FLOWCHART_INTERNAL_STORAGE",
            66,
            "flowChartInternalStorage",
            "Internal storage flowchart symbol",
        ),
        XmlMappedEnumMember(
            "FLOWCHART_MAGNETIC_DISK",
            86,
            "flowChartMagneticDisk",
            "Magnetic disk flowchart symbol",
        ),
        XmlMappedEnumMember(
            "FLOWCHART_MANUAL_INPUT",
            71,
            "flowChartManualInput",
            "Manual input flowchart symbol",
        ),
        XmlMappedEnumMember(
            "FLOWCHART_MANUAL_OPERATION",
            72,
            "flowChartManualOperation",
            "Manual operation flowchart symbol",
        ),
        XmlMappedEnumMember(
            "FLOWCHART_MERGE", 82, "flowChartMerge", "Merge flowchart symbol"
        ),
        XmlMappedEnumMember(
            "FLOWCHART_MULTIDOCUMENT",
            68,
            "flowChartMultidocument",
            "Multi-document flowchart symbol",
        ),
        XmlMappedEnumMember(
            "FLOWCHART_OFFLINE_STORAGE",
            139,
            "flowChartOfflineStorage",
            "Offline Storage",
        ),
        XmlMappedEnumMember(
            "FLOWCHART_OFFPAGE_CONNECTOR",
            74,
            "flowChartOffpageConnector",
            "Off-page connector flowchart symbol",
        ),
        XmlMappedEnumMember("FLOWCHART_OR", 78, "flowChartOr", '"Or" flowchart symbol'),
        XmlMappedEnumMember(
            "FLOWCHART_PREDEFINED_PROCESS",
            65,
            "flowChartPredefinedProcess",
            "Predefined process flowchart symbol",
        ),
        XmlMappedEnumMember(
            "FLOWCHART_PREPARATION",
            70,
            "flowChartPreparation",
            "Preparation flowchart symbol",
        ),
        XmlMappedEnumMember(
            "FLOWCHART_PROCESS", 61, "flowChartProcess", "Process flowchart symbol"
        ),
        XmlMappedEnumMember(
            "FLOWCHART_PUNCHED_TAPE",
            76,
            "flowChartPunchedTape",
            "Punched tape flowchart symbol",
        ),
        XmlMappedEnumMember(
            "FLOWCHART_SEQUENTIAL_ACCESS_STORAGE",
            85,
            "flowChartMagneticTape",
            "Sequential access storage flowchart symbol",
        ),
        XmlMappedEnumMember(
            "FLOWCHART_SORT", 80, "flowChartSort", "Sort flowchart symbol"
        ),
        XmlMappedEnumMember(
            "FLOWCHART_STORED_DATA",
            83,
            "flowChartOnlineStorage",
            "Stored data flowchart symbol",
        ),
        XmlMappedEnumMember(
            "FLOWCHART_SUMMING_JUNCTION",
            77,
            "flowChartSummingJunction",
            "Summing junction flowchart symbol",
        ),
        XmlMappedEnumMember(
            "FLOWCHART_TERMINATOR",
            69,
            "flowChartTerminator",
            "Terminator flowchart symbol",
        ),
        XmlMappedEnumMember("FOLDED_CORNER", 16, "foldedCorner", "Folded corner"),
        XmlMappedEnumMember("FRAME", 158, "frame", "Frame"),
        XmlMappedEnumMember("FUNNEL", 174, "funnel", "Funnel"),
        XmlMappedEnumMember("GEAR_6", 172, "gear6", "Gear 6"),
        XmlMappedEnumMember("GEAR_9", 173, "gear9", "Gear 9"),
        XmlMappedEnumMember("HALF_FRAME", 159, "halfFrame", "Half Frame"),
        XmlMappedEnumMember("HEART", 21, "heart", "Heart"),
        XmlMappedEnumMember("HEPTAGON", 145, "heptagon", "Heptagon"),
        XmlMappedEnumMember("HEXAGON", 10, "hexagon", "Hexagon"),
        XmlMappedEnumMember(
            "HORIZONTAL_SCROLL", 102, "horizontalScroll", "Horizontal scroll"
        ),
        XmlMappedEnumMember("ISOSCELES_TRIANGLE", 7, "triangle", "Isosceles triangle"),
        XmlMappedEnumMember(
            "LEFT_ARROW", 34, "leftArrow", "Block arrow that points left"
        ),
        XmlMappedEnumMember(
            "LEFT_ARROW_CALLOUT",
            54,
            "leftArrowCallout",
            "Callout with arrow that points left",
        ),
        XmlMappedEnumMember("LEFT_BRACE", 31, "leftBrace", "Left brace"),
        XmlMappedEnumMember("LEFT_BRACKET", 29, "leftBracket", "Left bracket"),
        XmlMappedEnumMember(
            "LEFT_CIRCULAR_ARROW", 176, "leftCircularArrow", "Left Circular Arrow"
        ),
        XmlMappedEnumMember(
            "LEFT_RIGHT_ARROW",
            37,
            "leftRightArrow",
            "Block arrow with arrowheads that point both left and right",
        ),
        XmlMappedEnumMember(
            "LEFT_RIGHT_ARROW_CALLOUT",
            57,
            "leftRightArrowCallout",
            "Callout with arrowheads that point both left and right",
        ),
        XmlMappedEnumMember(
            "LEFT_RIGHT_CIRCULAR_ARROW",
            177,
            "leftRightCircularArrow",
            "Left Right Circular Arrow",
        ),
        XmlMappedEnumMember(
            "LEFT_RIGHT_RIBBON", 140, "leftRightRibbon", "Left Right Ribbon"
        ),
        XmlMappedEnumMember(
            "LEFT_RIGHT_UP_ARROW",
            40,
            "leftRightUpArrow",
            "Block arrow with arrowheads that point left, right, and up",
        ),
        XmlMappedEnumMember(
            "LEFT_UP_ARROW",
            43,
            "leftUpArrow",
            "Block arrow with arrowheads that point left and up",
        ),
        XmlMappedEnumMember("LIGHTNING_BOLT", 22, "lightningBolt", "Lightning bolt"),
        XmlMappedEnumMember(
            "LINE_CALLOUT_1",
            109,
            "borderCallout1",
            "Callout with border and horizontal callout line",
        ),
        XmlMappedEnumMember(
            "LINE_CALLOUT_1_ACCENT_BAR",
            113,
            "accentCallout1",
            "Callout with vertical accent bar",
        ),
        XmlMappedEnumMember(
            "LINE_CALLOUT_1_BORDER_AND_ACCENT_BAR",
            121,
            "accentBorderCallout1",
            "Callout with border and vertical accent bar",
        ),
        XmlMappedEnumMember(
            "LINE_CALLOUT_1_NO_BORDER", 117, "callout1", "Callout with horizontal line"
        ),
        XmlMappedEnumMember(
            "LINE_CALLOUT_2",
            110,
            "borderCallout2",
            "Callout with diagonal straight line",
        ),
        XmlMappedEnumMember(
            "LINE_CALLOUT_2_ACCENT_BAR",
            114,
            "accentCallout2",
            "Callout with diagonal callout line and accent bar",
        ),
        XmlMappedEnumMember(
            "LINE_CALLOUT_2_BORDER_AND_ACCENT_BAR",
            122,
            "accentBorderCallout2",
            "Callout with border, diagonal straight line, and accent bar",
        ),
        XmlMappedEnumMember(
            "LINE_CALLOUT_2_NO_BORDER",
            118,
            "callout2",
            "Callout with no border and diagonal callout line",
        ),
        XmlMappedEnumMember(
            "LINE_CALLOUT_3", 111, "borderCallout3", "Callout with angled line"
        ),
        XmlMappedEnumMember(
            "LINE_CALLOUT_3_ACCENT_BAR",
            115,
            "accentCallout3",
            "Callout with angled callout line and accent bar",
        ),
        XmlMappedEnumMember(
            "LINE_CALLOUT_3_BORDER_AND_ACCENT_BAR",
            123,
            "accentBorderCallout3",
            "Callout with border, angled callout line, and accent bar",
        ),
        XmlMappedEnumMember(
            "LINE_CALLOUT_3_NO_BORDER",
            119,
            "callout3",
            "Callout with no border and angled callout line",
        ),
        XmlMappedEnumMember(
            "LINE_CALLOUT_4",
            112,
            "borderCallout3",
            "Callout with callout line segments forming a U-shape.",
        ),
        XmlMappedEnumMember(
            "LINE_CALLOUT_4_ACCENT_BAR",
            116,
            "accentCallout3",
            "Callout with accent bar and callout line segments forming a U-s" "hape.",
        ),
        XmlMappedEnumMember(
            "LINE_CALLOUT_4_BORDER_AND_ACCENT_BAR",
            124,
            "accentBorderCallout3",
            "Callout with border, accent bar, and callout line segments form"
            "ing a U-shape.",
        ),
        XmlMappedEnumMember(
            "LINE_CALLOUT_4_NO_BORDER",
            120,
            "callout3",
            "Callout with no border and callout line segments forming a U-sh" "ape.",
        ),
        XmlMappedEnumMember("LINE_INVERSE", 183, "lineInv", "Straight Connector"),
        XmlMappedEnumMember("MATH_DIVIDE", 166, "mathDivide", "Division"),
        XmlMappedEnumMember("MATH_EQUAL", 167, "mathEqual", "Equal"),
        XmlMappedEnumMember("MATH_MINUS", 164, "mathMinus", "Minus"),
        XmlMappedEnumMember("MATH_MULTIPLY", 165, "mathMultiply", "Multiply"),
        XmlMappedEnumMember("MATH_NOT_EQUAL", 168, "mathNotEqual", "Not Equal"),
        XmlMappedEnumMember("MATH_PLUS", 163, "mathPlus", "Plus"),
        XmlMappedEnumMember("MOON", 24, "moon", "Moon"),
        XmlMappedEnumMember(
            "NON_ISOSCELES_TRAPEZOID",
            143,
            "nonIsoscelesTrapezoid",
            "Non-isosceles Trapezoid",
        ),
        XmlMappedEnumMember(
            "NOTCHED_RIGHT_ARROW",
            50,
            "notchedRightArrow",
            "Notched block arrow that points right",
        ),
        XmlMappedEnumMember("NO_SYMBOL", 19, "noSmoking", '"No" symbol'),
        XmlMappedEnumMember("OCTAGON", 6, "octagon", "Octagon"),
        XmlMappedEnumMember("OVAL", 9, "ellipse", "Oval"),
        XmlMappedEnumMember(
            "OVAL_CALLOUT", 107, "wedgeEllipseCallout", "Oval-shaped callout"
        ),
        XmlMappedEnumMember("PARALLELOGRAM", 2, "parallelogram", "Parallelogram"),
        XmlMappedEnumMember("PENTAGON", 51, "homePlate", "Pentagon"),
        XmlMappedEnumMember("PIE", 142, "pie", "Pie"),
        XmlMappedEnumMember("PIE_WEDGE", 175, "pieWedge", "Pie"),
        XmlMappedEnumMember("PLAQUE", 28, "plaque", "Plaque"),
        XmlMappedEnumMember("PLAQUE_TABS", 171, "plaqueTabs", "Plaque Tabs"),
        XmlMappedEnumMember(
            "QUAD_ARROW",
            39,
            "quadArrow",
            "Block arrows that point up, down, left, and right",
        ),
        XmlMappedEnumMember(
            "QUAD_ARROW_CALLOUT",
            59,
            "quadArrowCallout",
            "Callout with arrows that point up, down, left, and right",
        ),
        XmlMappedEnumMember("RECTANGLE", 1, "rect", "Rectangle"),
        XmlMappedEnumMember(
            "RECTANGULAR_CALLOUT", 105, "wedgeRectCallout", "Rectangular callout"
        ),
        XmlMappedEnumMember("REGULAR_PENTAGON", 12, "pentagon", "Pentagon"),
        XmlMappedEnumMember(
            "RIGHT_ARROW", 33, "rightArrow", "Block arrow that points right"
        ),
        XmlMappedEnumMember(
            "RIGHT_ARROW_CALLOUT",
            53,
            "rightArrowCallout",
            "Callout with arrow that points right",
        ),
        XmlMappedEnumMember("RIGHT_BRACE", 32, "rightBrace", "Right brace"),
        XmlMappedEnumMember("RIGHT_BRACKET", 30, "rightBracket", "Right bracket"),
        XmlMappedEnumMember("RIGHT_TRIANGLE", 8, "rtTriangle", "Right triangle"),
        XmlMappedEnumMember("ROUNDED_RECTANGLE", 5, "roundRect", "Rounded rectangle"),
        XmlMappedEnumMember(
            "ROUNDED_RECTANGULAR_CALLOUT",
            106,
            "wedgeRoundRectCallout",
            "Rounded rectangle-shaped callout",
        ),
        XmlMappedEnumMember(
            "ROUND_1_RECTANGLE", 151, "round1Rect", "Round Single Corner Rectangle"
        ),
        XmlMappedEnumMember(
            "ROUND_2_DIAG_RECTANGLE",
            153,
            "round2DiagRect",
            "Round Diagonal Corner Rectangle",
        ),
        XmlMappedEnumMember(
            "ROUND_2_SAME_RECTANGLE",
            152,
            "round2SameRect",
            "Round Same Side Corner Rectangle",
        ),
        XmlMappedEnumMember("SMILEY_FACE", 17, "smileyFace", "Smiley face"),
        XmlMappedEnumMember(
            "SNIP_1_RECTANGLE", 155, "snip1Rect", "Snip Single Corner Rectangle"
        ),
        XmlMappedEnumMember(
            "SNIP_2_DIAG_RECTANGLE",
            157,
            "snip2DiagRect",
            "Snip Diagonal Corner Rectangle",
        ),
        XmlMappedEnumMember(
            "SNIP_2_SAME_RECTANGLE",
            156,
            "snip2SameRect",
            "Snip Same Side Corner Rectangle",
        ),
        XmlMappedEnumMember(
            "SNIP_ROUND_RECTANGLE",
            154,
            "snipRoundRect",
            "Snip and Round Single Corner Rectangle",
        ),
        XmlMappedEnumMember("SQUARE_TABS", 170, "squareTabs", "Square Tabs"),
        XmlMappedEnumMember("STAR_10_POINT", 149, "star10", "10-Point Star"),
        XmlMappedEnumMember("STAR_12_POINT", 150, "star12", "12-Point Star"),
        XmlMappedEnumMember("STAR_16_POINT", 94, "star16", "16-point star"),
        XmlMappedEnumMember("STAR_24_POINT", 95, "star24", "24-point star"),
        XmlMappedEnumMember("STAR_32_POINT", 96, "star32", "32-point star"),
        XmlMappedEnumMember("STAR_4_POINT", 91, "star4", "4-point star"),
        XmlMappedEnumMember("STAR_5_POINT", 92, "star5", "5-point star"),
        XmlMappedEnumMember("STAR_6_POINT", 147, "star6", "6-Point Star"),
        XmlMappedEnumMember("STAR_7_POINT", 148, "star7", "7-Point Star"),
        XmlMappedEnumMember("STAR_8_POINT", 93, "star8", "8-point star"),
        XmlMappedEnumMember(
            "STRIPED_RIGHT_ARROW",
            49,
            "stripedRightArrow",
            "Block arrow that points right with stripes at the tail",
        ),
        XmlMappedEnumMember("SUN", 23, "sun", "Sun"),
        XmlMappedEnumMember("SWOOSH_ARROW", 178, "swooshArrow", "Swoosh Arrow"),
        XmlMappedEnumMember("TEAR", 160, "teardrop", "Teardrop"),
        XmlMappedEnumMember("TRAPEZOID", 3, "trapezoid", "Trapezoid"),
        XmlMappedEnumMember("UP_ARROW", 35, "upArrow", "Block arrow that points up"),
        XmlMappedEnumMember(
            "UP_ARROW_CALLOUT",
            55,
            "upArrowCallout",
            "Callout with arrow that points up",
        ),
        XmlMappedEnumMember(
            "UP_DOWN_ARROW", 38, "upDownArrow", "Block arrow that points up and down"
        ),
        XmlMappedEnumMember(
            "UP_DOWN_ARROW_CALLOUT",
            58,
            "upDownArrowCallout",
            "Callout with arrows that point up and down",
        ),
        XmlMappedEnumMember(
            "UP_RIBBON",
            97,
            "ribbon2",
            "Ribbon banner with center area above ribbon ends",
        ),
        XmlMappedEnumMember(
            "U_TURN_ARROW", 42, "uturnArrow", "Block arrow forming a U shape"
        ),
        XmlMappedEnumMember(
            "VERTICAL_SCROLL", 101, "verticalScroll", "Vertical scroll"
        ),
        XmlMappedEnumMember("WAVE", 103, "wave", "Wave"),
    )


@alias("MSO_CONNECTOR")
class MSO_CONNECTOR_TYPE(XmlEnumeration):
    """
    Specifies a type of connector.

    Alias: ``MSO_CONNECTOR``

    Example::

        from pptx.enum.shapes import MSO_CONNECTOR
        from pptx.util import Cm

        shapes = prs.slides[0].shapes
        connector = shapes.add_connector(
            MSO_CONNECTOR.STRAIGHT, Cm(2), Cm(2), Cm(10), Cm(10)
        )
        assert connector.left.cm == 2
    """

    __ms_name__ = "MsoConnectorType"

    __url__ = "http://msdn.microsoft.com/en-us/library/office/ff860918.aspx"

    __members__ = (
        XmlMappedEnumMember("CURVE", 3, "curvedConnector3", "Curved connector."),
        XmlMappedEnumMember("ELBOW", 2, "bentConnector3", "Elbow connector."),
        XmlMappedEnumMember("STRAIGHT", 1, "line", "Straight line connector."),
        ReturnValueOnlyEnumMember(
            "MIXED",
            -2,
            "Return value only; indicates a combination of othe" "r states.",
        ),
    )


@alias("MSO")
class MSO_SHAPE_TYPE(Enumeration):
    """
    Specifies the type of a shape

    Alias: ``MSO``

    Example::

        from pptx.enum.shapes import MSO_SHAPE_TYPE

        assert shape.type == MSO_SHAPE_TYPE.PICTURE
    """

    __ms_name__ = "MsoShapeType"

    __url__ = (
        "http://msdn.microsoft.com/en-us/library/office/ff860759(v=office.15" ").aspx"
    )

    __members__ = (
        EnumMember("AUTO_SHAPE", 1, "AutoShape"),
        EnumMember("CALLOUT", 2, "Callout shape"),
        EnumMember("CANVAS", 20, "Drawing canvas"),
        EnumMember("CHART", 3, "Chart, e.g. pie chart, bar chart"),
        EnumMember("COMMENT", 4, "Comment"),
        EnumMember("DIAGRAM", 21, "Diagram"),
        EnumMember("EMBEDDED_OLE_OBJECT", 7, "Embedded OLE object"),
        EnumMember("FORM_CONTROL", 8, "Form control"),
        EnumMember("FREEFORM", 5, "Freeform"),
        EnumMember("GROUP", 6, "Group shape"),
        EnumMember("IGX_GRAPHIC", 24, "SmartArt graphic"),
        EnumMember("INK", 22, "Ink"),
        EnumMember("INK_COMMENT", 23, "Ink Comment"),
        EnumMember("LINE", 9, "Line"),
        EnumMember("LINKED_OLE_OBJECT", 10, "Linked OLE object"),
        EnumMember("LINKED_PICTURE", 11, "Linked picture"),
        EnumMember("MEDIA", 16, "Media"),
        EnumMember("OLE_CONTROL_OBJECT", 12, "OLE control object"),
        EnumMember("PICTURE", 13, "Picture"),
        EnumMember("PLACEHOLDER", 14, "Placeholder"),
        EnumMember("SCRIPT_ANCHOR", 18, "Script anchor"),
        EnumMember("TABLE", 19, "Table"),
        EnumMember("TEXT_BOX", 17, "Text box"),
        EnumMember("TEXT_EFFECT", 15, "Text effect"),
        EnumMember("WEB_VIDEO", 26, "Web video"),
        ReturnValueOnlyEnumMember("MIXED", -2, "Mixed shape types"),
    )


class PP_MEDIA_TYPE(Enumeration):
    """
    Indicates the OLE media type.

    Example::

        from pptx.enum.shapes import PP_MEDIA_TYPE

        movie = slide.shapes[0]
        assert movie.media_type == PP_MEDIA_TYPE.MOVIE
    """

    __ms_name__ = "PpMediaType"

    __url__ = "https://msdn.microsoft.com/en-us/library/office/ff746008.aspx"

    __members__ = (
        EnumMember("MOVIE", 3, "Video media such as MP4."),
        EnumMember("OTHER", 1, "Other media types"),
        EnumMember("SOUND", 1, "Audio media such as MP3."),
        ReturnValueOnlyEnumMember(
            "MIXED",
            -2,
            "Return value only; indicates multiple media types,"
            " typically for a collection of shapes. May not be applicable in"
            " python-pptx.",
        ),
    )


@alias("PP_PLACEHOLDER")
class PP_PLACEHOLDER_TYPE(XmlEnumeration):
    """
    Specifies one of the 18 distinct types of placeholder.

    Alias: ``PP_PLACEHOLDER``

    Example::

        from pptx.enum.shapes import PP_PLACEHOLDER

        placeholder = slide.placeholders[0]
        assert placeholder.type == PP_PLACEHOLDER.TITLE
    """

    __ms_name__ = "PpPlaceholderType"

    __url__ = (
        "http://msdn.microsoft.com/en-us/library/office/ff860759(v=office.15" ").aspx"
    )

    __members__ = (
        XmlMappedEnumMember("BITMAP", 9, "clipArt", "Clip art placeholder"),
        XmlMappedEnumMember("BODY", 2, "body", "Body"),
        XmlMappedEnumMember("CENTER_TITLE", 3, "ctrTitle", "Center Title"),
        XmlMappedEnumMember("CHART", 8, "chart", "Chart"),
        XmlMappedEnumMember("DATE", 16, "dt", "Date"),
        XmlMappedEnumMember("FOOTER", 15, "ftr", "Footer"),
        XmlMappedEnumMember("HEADER", 14, "hdr", "Header"),
        XmlMappedEnumMember("MEDIA_CLIP", 10, "media", "Media Clip"),
        XmlMappedEnumMember("OBJECT", 7, "obj", "Object"),
        XmlMappedEnumMember(
            "ORG_CHART",
            11,
            "dgm",
            "SmartArt placeholder. Organization char" "t is a legacy name.",
        ),
        XmlMappedEnumMember("PICTURE", 18, "pic", "Picture"),
        XmlMappedEnumMember("SLIDE_IMAGE", 101, "sldImg", "Slide Image"),
        XmlMappedEnumMember("SLIDE_NUMBER", 13, "sldNum", "Slide Number"),
        XmlMappedEnumMember("SUBTITLE", 4, "subTitle", "Subtitle"),
        XmlMappedEnumMember("TABLE", 12, "tbl", "Table"),
        XmlMappedEnumMember("TITLE", 1, "title", "Title"),
        ReturnValueOnlyEnumMember("VERTICAL_BODY", 6, "Vertical Body"),
        ReturnValueOnlyEnumMember("VERTICAL_OBJECT", 17, "Vertical Object"),
        ReturnValueOnlyEnumMember("VERTICAL_TITLE", 5, "Vertical Title"),
        ReturnValueOnlyEnumMember(
            "MIXED",
            -2,
            "Return value only; multiple placeholders of differ" "ing types.",
        ),
    )


class _ProgIdEnum(object):
    """One-off Enum-like object for progId values.

    Indicates the type of an OLE object in terms of the program used to open it.

    A member of this enumeration can be used in a `SlideShapes.add_ole_object()` call to
    specify a Microsoft Office file-type (Excel, PowerPoint, or Word), which will
    then not require several of the arguments required to embed other object types.

    Example::

        from pptx.enum.shapes import PROG_ID
        from pptx.util import Inches

        embedded_xlsx_shape = slide.shapes.add_ole_object(
            "workbook.xlsx", PROG_ID.XLSX, left=Inches(1), top=Inches(1)
        )
        assert embedded_xlsx_shape.ole_format.prog_id == "Excel.Sheet.12"
    """

    class Member(object):
        """A particular progID with its attributes."""

        def __init__(self, name, progId, icon_filename, width, height):
            self._name = name
            self._progId = progId
            self._icon_filename = icon_filename
            self._width = width
            self._height = height

        def __repr__(self):
            return "PROG_ID.%s" % self._name

        @property
        def height(self):
            return self._height

        @property
        def icon_filename(self):
            return self._icon_filename

        @property
        def progId(self):
            return self._progId

        @property
        def width(self):
            return self._width

    def __contains__(self, item):
        return item in (
            self.DOCX,
            self.PPTX,
            self.XLSX,
        )

    def __repr__(self):
        return "%s.PROG_ID" % __name__

    @lazyproperty
    def DOCX(self):
        return self.Member("DOCX", "Word.Document.12", "docx-icon.emf", 965200, 609600)

    @lazyproperty
    def PPTX(self):
        return self.Member(
            "PPTX", "PowerPoint.Show.12", "pptx-icon.emf", 965200, 609600
        )

    @lazyproperty
    def XLSX(self):
        return self.Member("XLSX", "Excel.Sheet.12", "xlsx-icon.emf", 965200, 609600)


PROG_ID = _ProgIdEnum()
